/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Adjuster.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Adjuster *) SCM_CDR (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Adjuster) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Adjuster *widget = (Fl_Adjuster *) SCM_CDR (widget_smob);
  scm_puts ("#<widget Fl_Adjuster ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-adjuster", 
		   "The fltk-adjuster is used to adjust ranged values.  When "
		   "you click a button and drag to the left, the value "
		   "decreases and when you drag to the right, the value "
		   "increases.  The first mouse button increments by the "
		   "step value; the second mouse button increments by 10 * "
		   "step; the third mouse button increments by 100 * step.",
		   fltk_adjuster, fltk_valuator, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1
( fltk_adjuster_new ,
  "fltk-adjuster::new",
  "Creates a new fltk-adjuster with the given position, size and "
  "label string",
  x, y, w, h, label,
  int, int, int, int, str,
  widget,
  fltk_adjuster)
{
  GFL_ASSERT_5 (fltk_adjuster_new, x, y, w, h, label);
  Fl_Adjuster *widget = new Fl_Adjuster(gh_scm2int(x), gh_scm2int(y), 
					gh_scm2int(w), gh_scm2int(h),
					gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_adjuster, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
 ( fltk_adjuster_get_soft, 
   "fltk-adjuster::get-soft", 
   "Sets wether the user is allowed to drag the value outside the range.", 
   widget, 
   fltk_adjuster, 
   soft, 
   bool)
{
  GFL_ASSERT_1(fltk_adjuster_get_soft, widget); 
  Fl_Adjuster *wi = (Fl_Adjuster *) SCM_CDR (widget);
  if (gh_int2scm((uchar)wi->soft()))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
  ( fltk_adjuster_set_soft, 
   "fltk-adjuster::set-soft", 
   "Gets wether the user is allowed to drag the value outside the range.", 
   widget, soft, 
   fltk_adjuster, bool)
{
  GFL_ASSERT_2(fltk_adjuster_set_soft, widget, soft);
  Fl_Adjuster *wi = (Fl_Adjuster *) SCM_CDR (widget);
  wi->soft((uchar)gh_scm2bool(soft));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_adjuster_init ()
{
  GFL_NEW_TYPE (fltk_adjuster);
  GFL_NEW_PROCEDURE (fltk_adjuster_new);
  GFL_NEW_PROCEDURE (fltk_adjuster_get_soft);
  GFL_NEW_PROCEDURE (fltk_adjuster_set_soft);
}
