/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Browser_.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Browser_ *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Browser_) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Browser_ *widget = (Fl_Browser_ *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Browser_ ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-browser-base",
		   "fltk-browser-base is the base class for browsers.",
		   fltk_browser_base, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_2_0
( fltk_browser_base_has_scrollbar, 
  "fltk-browser-base::has-scrollbar", 
  "Sets the type of scrollbar.  Valid options are "
  "FL_BROWSER_NONE, FL_BROWSER_HORIZONTAL, FL_BROWSER_VERTICAL, "
  "FL_BROWSER_BOTH, FL_BROWSER_HORIZONTAL_ALWAYS, FL_BROWSER_VERTICAL_ALWAYS, "
  "and FL_BROWSER_BOTH_ALWAYS.",
  widget, a, 
  fltk_browser_base, int)
{
  GFL_ASSERT_2(fltk_browser_base_has_scrollbar, widget, a);
  Fl_Browser_ *wi = (Fl_Browser_ *) gh_cdr (widget);
  wi->has_scrollbar(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_browser_base_set_text_color, 
  "fltk-browser-base::set-text-color", 
  "Sets the default text color for lines in the browser.", 
  widget, a, 
  fltk_browser_base, int)
{
  GFL_ASSERT_2(fltk_browser_base_set_text_color, widget, a);
  Fl_Browser_ *wi = (Fl_Browser_ *) gh_cdr (widget);
  wi->textcolor(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_browser_base_get_text_color, 
  "fltk-browser-base::get-text-color", 
  "Gets the default text color for lines in the browser.", 
  widget, 
  fltk_browser_base, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_browser_base_get_text_color, widget);
  Fl_Browser_ *wi = (Fl_Browser_ *) gh_cdr (widget);
  return gh_int2scm(wi->textcolor());
}

GFL_PROCEDURE_2_0
( fltk_browser_base_set_text_font, 
  "fltk-browser-base::set-text-font", 
  "Sets the default font for text in the browser.", 
  widget, a, 
  fltk_browser_base, int)
{
  GFL_ASSERT_2(fltk_browser_base_set_text_font, widget, a);
  Fl_Browser_ *wi = (Fl_Browser_ *) gh_cdr (widget);
  wi->textfont(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_browser_base_get_text_font, 
  "fltk-browser-base::get-text-font", 
  "Gets the default font for text in the browser.", 
  widget, 
  fltk_browser_base, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_browser_base_get_text_font, widget);
  
    Fl_Browser_ *wi = (Fl_Browser_ *) gh_cdr (widget);
  return gh_int2scm(wi->textfont());
}

GFL_PROCEDURE_2_0
( fltk_browser_base_set_text_size, 
  "fltk-browser-base::set-text-size", 
  "Sets the default font size for text in the browser.", 
  widget, a, 
  fltk_browser_base, int)
{
  GFL_ASSERT_2(fltk_browser_base_set_text_size, widget, a);
  Fl_Browser_ *wi = (Fl_Browser_ *) gh_cdr (widget);
  wi->textsize(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_browser_base_get_text_size, 
  "fltk-browser-base::get-text-size", 
  "Gets the default font size for text in the browser.", 
  widget, 
  fltk_browser_base, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_browser_base_get_text_size, widget);
  Fl_Browser_ *wi = (Fl_Browser_ *) gh_cdr (widget);
  return gh_int2scm(wi->textsize());
}

/*
   4.  Initialization Hook:
*/

void
fltk_browser_base_init ()
{
  GFL_NEW_TYPE (fltk_browser_base);
  GFL_NEW_PROCEDURE (fltk_browser_base_has_scrollbar);
  GFL_NEW_PROCEDURE (fltk_browser_base_set_text_color);
  GFL_NEW_PROCEDURE (fltk_browser_base_get_text_color);
  GFL_NEW_PROCEDURE (fltk_browser_base_set_text_font);
  GFL_NEW_PROCEDURE (fltk_browser_base_get_text_font);
  GFL_NEW_PROCEDURE (fltk_browser_base_set_text_size);
  GFL_NEW_PROCEDURE (fltk_browser_base_get_text_size);
}

