/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Button.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Button *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Button) + sizeof (Fl_Button));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Button *widget = (Fl_Button *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Button ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-button",
		   "fltk-button generates a callback when it is clicked by a user.",
		   fltk_button, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_button_new,
  "fltk-button::new",
  "Creates a new fltk-button with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  button,
  fltk_button )
{
  GFL_ASSERT_5 (fltk_button_new, x, y, w, h, label);
  Fl_Button * widget = new Fl_Button(gh_scm2int(x), gh_scm2int(y), 
				     gh_scm2int(w), gh_scm2int(h),
				     gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_button, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_0
( fltk_button_clear, 
  "fltk-button::clear", 
  "Sets the value to 0 (off).", 
  widget, 
 fltk_button )
{
  GFL_ASSERT_1(fltk_button_clear, widget);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->clear();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_button_set_down_box, 
  "fltk-button::set-down-box", 
  "Sets the boxtype drawn when a button is clicked \"down\".", 
  widget, down_box, 
  fltk_button, int)
{
  GFL_ASSERT_2(fltk_button_set_down_box, widget, down_box);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->down_box((Fl_Boxtype)gh_scm2int(down_box));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_button_get_down_box, 
  "fltk-button::get-down-box", 
  "Gets the boxtype drawn when a button is clicked \"down\".", 
  widget, 
  fltk_button, 
  down_box, 
  int)
{
  GFL_ASSERT_1(fltk_button_get_down_box, widget);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  return gh_int2scm((int)wi->down_box());
}

GFL_PROCEDURE_1_0
( fltk_button_set, 
  "fltk-button::set", 
  "Sets the value to 1 (on).", 
  widget, 
 fltk_button )
{
  GFL_ASSERT_1(fltk_button_set, widget);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->set();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_button_setonly, 
  "fltk-button::setonly", 
  "Sets the value to 1 (on), and sets all other radio buttons in this group "
  "to 0 (off).",
  widget, 
 fltk_button )
{
  GFL_ASSERT_1(fltk_button_setonly, widget);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->setonly();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_button_set_shortcut, 
  "fltk-button::set-shortcut", 
  "Sets the shortcut key for this button.", 
  widget, a, 
  fltk_button, int)
{
  GFL_ASSERT_2(fltk_button_set_shortcut, widget, a);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->shortcut(gh_scm2long(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_button_get_shortcut, 
  "fltk-button::get-shortcut", 
  "Gets the shortcut key for this button.", 
  widget, 
  fltk_button, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_button_get_shortcut, widget);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  return gh_int2scm((int)wi->shortcut());
}

GFL_PROCEDURE_2_0
( fltk_button_set_type, 
  "fltk-button::set-type", 
  "Sets the button type.  Valid values include: 0, FL_TOGGLE_BUTTON, "
  "and FL_RADIO_BUTTON.", 
  widget, a, 
  fltk_button, int)
{
  GFL_ASSERT_2(fltk_button_set_type, widget, a);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->type(gh_scm2long(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_button_get_type, 
  "fltk-button::get-type", 
  "Gets the button type.", 
  widget, 
  fltk_button, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_button_get_type, widget);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  return gh_int2scm((int)wi->type());
}

GFL_PROCEDURE_2_0
( fltk_button_set_value, 
  "fltk-button::set-value", 
  "Sets the current value (0 or 1).", 
  widget, a, 
  fltk_button, int)
{
  GFL_ASSERT_2(fltk_button_set_value, widget, a);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->type(gh_scm2long(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_button_get_value, 
  "fltk-button::get-value", 
  "Gets the current value (0 or 1).", 
  widget, 
  fltk_button, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_button_get_value, widget);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  return gh_int2scm((int)wi->value());
}

GFL_PROCEDURE_2_0
( fltk_button_set_when, 
  "fltk-button::set-when", 
  "Sets when callbacks are done.  Valid values include: FL_NONE, FL_WHEN_RELEASE, and FL_WHEN_CHANGE.", 
  widget, a, 
  fltk_button, int)
{
  GFL_ASSERT_2(fltk_button_set_when, widget, a);
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  wi->when(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_button_get_when, 
  "fltk-button::get-when", 
  "Gets when callbacks are done.", 
  widget, 
  fltk_button, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_button_get_when, widget);
  
  Fl_Button *wi = (Fl_Button *) gh_cdr (widget);
  return gh_int2scm((int)wi->when());
}

/*
   4.  Initialization Hook:
*/

void
fltk_button_init ()
{
  GFL_NEW_TYPE (fltk_button);
  GFL_NEW_PROCEDURE (fltk_button_new);
  GFL_NEW_PROCEDURE (fltk_button_clear);
  GFL_NEW_PROCEDURE (fltk_button_set_down_box);
  GFL_NEW_PROCEDURE (fltk_button_get_down_box);
  GFL_NEW_PROCEDURE (fltk_button_set);
  GFL_NEW_PROCEDURE (fltk_button_setonly);
  GFL_NEW_PROCEDURE (fltk_button_set_shortcut);
  GFL_NEW_PROCEDURE (fltk_button_get_shortcut);
  GFL_NEW_PROCEDURE (fltk_button_set_type);
  GFL_NEW_PROCEDURE (fltk_button_get_type);
  GFL_NEW_PROCEDURE (fltk_button_set_value);
  GFL_NEW_PROCEDURE (fltk_button_get_value);
  GFL_NEW_PROCEDURE (fltk_button_set_when);
  GFL_NEW_PROCEDURE (fltk_button_get_when);

  gh_define ("FL_TOGGLE_BUTTON", gh_long2scm(FL_TOGGLE_BUTTON));
  gh_define ("FL_RADIO_BUTTON", gh_long2scm(FL_RADIO_BUTTON));
  gh_define ("FL_HIDDEN_BUTTON", gh_long2scm(FL_HIDDEN_BUTTON));
}
