/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Chart.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Chart *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Chart) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Chart *widget = (Fl_Chart *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Chart ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

GFL_SUBCLASS_TYPE ("fltk-chart", 
		   "fltk-chart displays simple charts.",
		   fltk_chart, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_chart_new,
   "fltk-chart::new",
   "Creates a new fltk-chart with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   input,
   fltk_chart )
{
  GFL_ASSERT_5 (fltk_chart_new, x, y, w, h, label);

  Fl_Chart * widget = new Fl_Chart(gh_scm2int(x), gh_scm2int(y), 
				   gh_scm2int(w), gh_scm2int(h),
				   gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_chart, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_4_0
  ( fltk_chart_add,
   "fltk-chart::add",
   "Adds a value with label and color to the chart.",
   widget, value, label, color,
   fltk_chart, double, str, fltk_color)
{
  GFL_ASSERT_4 (fltk_chart_add, widget, value, label, color);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  w->add(gh_scm2double(value), gh_scm2newstr(label, NULL), gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_chart_get_autosize,
   "fltk-chart::get-autosize",
   "Gets wether or not the chart bounds are automatically adjusted.",
   widget, 
   fltk_chart,
   autosize,
   bool )
{
  GFL_ASSERT_1 (fltk_chart_get_autosize, widget);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  if (w->autosize())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
  ( fltk_chart_set_autosize,
   "fltk-chart::set-autosize",
   "Sets whether or not the chart bounds are automatically adjusted.",
   widget, autosize,
   fltk_chart, bool )
{
  GFL_ASSERT_2 (fltk_chart_set_autosize, widget, autosize);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  w->autosize((uchar)gh_scm2int(autosize));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
  ( fltk_chart_set_bounds,
   "fltk-chart::set-bounds",
   "Sets the lower and upper bounds of the chart.",
   widget, lower, upper,
   fltk_chart, double, double )
{
  GFL_ASSERT_3 (fltk_chart_set_bounds, widget, lower, upper);
  Fl_Chart * w = (Fl_Chart *) gh_cdr (widget);
  w->bounds(gh_scm2double(lower), gh_scm2double(upper));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_2
  ( fltk_chart_get_bounds,
   "fltk-chart::get-bounds",
   "Gets the lower and upper bounds of the chart.",
   widget, 
   fltk_chart, 
   lower, upper,
   double, double )
{
  GFL_ASSERT_1 (fltk_chart_get_bounds, widget);
  Fl_Chart * w = (Fl_Chart *) gh_cdr (widget);
  double a, b;
  w->bounds(&a, &b);
  return gh_cons ( gh_double2scm(a), gh_cons ( gh_double2scm(b), SCM_EOL));
}

GFL_PROCEDURE_1_0
  ( fltk_chart_clear,
   "fltk-chart::clear",
   "Clears the chart.",
   widget,
   fltk_chart )
{
  GFL_ASSERT_1 (fltk_chart_clear, widget);
  Fl_Chart * w = (Fl_Chart *) gh_cdr (widget);
  w->clear();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_5_0
  ( fltk_chart_insert,
   "fltk-chart::insert",
   "Inserts the data at the index \"position\" with the given value, label and color.",
   widget, position, value, label, color,
   fltk_chart, int, double, str, fltk_color )
{
  GFL_ASSERT_5 (fltk_chart_insert, widget, position, value, label, color);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  w->insert(gh_scm2int(position), gh_scm2double(value), gh_scm2newstr(label, NULL), 
    (uchar)gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_chart_get_max_size,
   "fltk-chart::get-max_size",
   "Gets the maximum size of the chart.",
   widget, 
   fltk_chart,
   max_size,
   double )
{
  GFL_ASSERT_1 (fltk_chart_get_max_size, widget);
  Fl_Chart * w = (Fl_Chart *) gh_cdr (widget);
  return gh_int2scm(w->maxsize());
}

GFL_PROCEDURE_2_0
  ( fltk_chart_set_max_size,
   "fltk-chart::set-max-size",
   "Sets the maximum chart size.",
   widget, max_size,
   fltk_chart, double )
{
  GFL_ASSERT_2 (fltk_chart_set_max_size, widget, max_size);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  w->maxsize (gh_scm2int (max_size));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_5_0
  ( fltk_chart_replace,
   "fltk-chart::replace",
   "Replaces the data at the index \"position\" with the given value, label and color.",
   widget, position, value, label, color,
   fltk_chart, int, double, str, fltk_color )
{
  GFL_ASSERT_5 (fltk_chart_insert, widget, position, value, label, color);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  w->replace(gh_scm2int(position), gh_scm2double(value), gh_scm2newstr(label, NULL), 
    (uchar)gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_chart_get_size,
   "fltk-chart::get-size",
   "Gets the size of the chart.",
   widget,
   fltk_chart,
   size,
   int)
{
  GFL_ASSERT_1 (fltk_chart_get_size, widget);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  return gh_int2scm(w->size());
}

GFL_PROCEDURE_1_1
  ( fltk_chart_get_type,
   "fltk-chart::get-type",
   "Gets the type of chart.",
   widget,
   fltk_chart,
   type,
   fltk_charttype)
{
  GFL_ASSERT_1 (fltk_chart_get_type, widget);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  return gh_int2scm((int)w->type());
}

GFL_PROCEDURE_2_0
  ( fltk_chart_set_type,
   "fltk-chart::set-type",
   "Sets the type of chart.",
   widget, type,
   fltk_chart, fltk_charttype)
{
  GFL_ASSERT_2 (fltk_chart_set_type, widget, type);
  Fl_Chart *w = (Fl_Chart *) gh_cdr (widget);
  w->type((uchar)gh_scm2int(type));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_chart_init ()
{
  GFL_NEW_TYPE (fltk_chart);
  GFL_NEW_PROCEDURE (fltk_chart_new);
  GFL_NEW_PROCEDURE (fltk_chart_add);
  GFL_NEW_PROCEDURE (fltk_chart_get_autosize);
  GFL_NEW_PROCEDURE (fltk_chart_set_autosize);
  GFL_NEW_PROCEDURE (fltk_chart_get_bounds);
  GFL_NEW_PROCEDURE (fltk_chart_set_bounds);
  GFL_NEW_PROCEDURE (fltk_chart_clear);
  GFL_NEW_PROCEDURE (fltk_chart_insert);
  GFL_NEW_PROCEDURE (fltk_chart_get_max_size);
  GFL_NEW_PROCEDURE (fltk_chart_set_max_size);
  GFL_NEW_PROCEDURE (fltk_chart_replace);
  GFL_NEW_PROCEDURE (fltk_chart_get_size);
  GFL_NEW_PROCEDURE (fltk_chart_get_type);
  GFL_NEW_PROCEDURE (fltk_chart_set_type);
}
