/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Choice.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Choice *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (GFL_Widget) + sizeof (Fl_Choice));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Choice *widget = (Fl_Choice *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Choice ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-choice", 
		   "fltk-choice is a button that when pushed pops up a menu.",
		   fltk_choice, fltk_menu, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_choice_new,
   "fltk-choice::new",
   "Creates a new fltk-choice with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   choice,
   fltk_choice )
{
  GFL_ASSERT_5 (fltk_choice_new, x, y, w, h, label);
  Fl_Choice * widget = new Fl_Choice(gh_scm2int(x), gh_scm2int(y),
			       gh_scm2int(w), gh_scm2int(h),
			       gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_choice, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
  ( fltk_choice_get_value,
   "fltk-choice::get-value",
   "Gets the index of the currently selected item.",
   widget,
   fltk_choice,
   value,
   int )
{
  GFL_ASSERT_1 (fltk_choice_get_value, widget);
  Fl_Choice *w = (Fl_Choice *) gh_cdr (widget);
  return gh_int2scm((int)w->value());
}

GFL_PROCEDURE_2_0
  ( fltk_choice_set_value,
   "fltk-choice::set-value",
   "Sets the currently selected items to index N.",
   widget, N,
   fltk_choice, int )
{
  GFL_ASSERT_2 (fltk_choice_set_value, widget, N);
  Fl_Choice * w = (Fl_Choice *) gh_cdr (widget);
  w->value(gh_scm2int(N));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_choice_get_changed,
    "fltk-choice::get-changed",
    "Gets wether the choice has been changed.",
    widget,
    fltk_choice,
    changed,
    bool)
{
  GFL_ASSERT_1 (fltk_choice_get_changed, widget);
  Fl_Choice * wi = (Fl_Choice *) gh_cdr (widget);
  if (wi->changed())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
  ( fltk_choice_set_changed,
    "fltk-choice::set-changed",
    "Sets wether the choice has been changed.",
    widget, changed,
    fltk_choice, bool )
{
  GFL_ASSERT_2 (fltk_choice_set_changed, widget, changed);
  Fl_Choice * wi = (Fl_Choice *) gh_cdr (widget);
  if (gh_scm2bool(changed))
    wi->set_changed();
  else
    wi->clear_changed();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_choice_get_down_box,
    "fltk-choice::get-down-box",
    "Gets the down box type for this widget.",
    widget,
    fltk_choice,
    down_box,
    fltk_boxtype)
{
  GFL_ASSERT_1 (fltk_choice_get_down_box, widget);
  Fl_Choice *wi = (Fl_Choice *) gh_cdr (widget);
  return gh_int2scm((int)wi->down_box());
}

GFL_PROCEDURE_2_0
  ( fltk_choice_set_down_box,
    "fltk-choice::get-down-box",
    "Sets the down box type for this widget.",
    widget, down_box,
    fltk_choice, fltk_boxtype)
{
  GFL_ASSERT_2 (fltk_choice_set_down_box, widget, down_box);
  Fl_Choice *wi = (Fl_Choice *) gh_cdr (widget);
  wi->down_box((Fl_Boxtype)gh_scm2int(down_box));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_choice_init ()
{
  GFL_NEW_TYPE (fltk_choice);
  GFL_NEW_PROCEDURE (fltk_choice_new);
  GFL_NEW_PROCEDURE (fltk_choice_get_value);
  GFL_NEW_PROCEDURE (fltk_choice_set_value);
  GFL_NEW_PROCEDURE (fltk_choice_set_changed);
  GFL_NEW_PROCEDURE (fltk_choice_get_changed);
  GFL_NEW_PROCEDURE (fltk_choice_get_down_box);
  GFL_NEW_PROCEDURE (fltk_choice_set_down_box);
}
