/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Clock.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Clock *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Clock) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Clock *widget = (Fl_Clock *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Clock ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-clock",
		   "fltk-clock provides a round analog clock display.",
		   fltk_clock, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_clock_new,
   "fltk-clock::new",
   "Creates a new fltk-clock with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   clock,
   fltk_clock )
{
  GFL_ASSERT_5 (fltk_clock_new, x, y, w, h, label);

  Fl_Clock * widget = new Fl_Clock(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_clock, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
  ( fltk_clock_hour, 
   "fltk-clock::hour", 
   "Returns the current hour.", 
   widget, 
   fltk_clock, 
   hour, 
   int)
{
  GFL_ASSERT_1(fltk_clock_hour, widget);  
  Fl_Clock *wi = (Fl_Clock *) gh_cdr (widget);
  return gh_int2scm(wi->hour());
}

GFL_PROCEDURE_1_1
  ( fltk_clock_minute, 
   "fltk-clock::minute", 
   "Returns the current minute.", 
   widget, 
   fltk_clock, 
   minute, 
   int)
{
  GFL_ASSERT_1(fltk_clock_minute, widget);
  Fl_Clock *wi = (Fl_Clock *) gh_cdr (widget);
  return gh_int2scm(wi->minute());
}


GFL_PROCEDURE_1_1
  ( fltk_clock_second, 
   "fltk-clock::second", 
   "Returns the current second.",
   widget, 
   fltk_clock, 
   second, 
   int)
{
  GFL_ASSERT_1(fltk_clock_second, widget);
  Fl_Clock *wi = (Fl_Clock *) gh_cdr (widget);
  return gh_int2scm(wi->second());
}


GFL_PROCEDURE_4_0
  ( fltk_clock_value, 
   "fltk-clock::value", 
   "Sets the current hour, minute and second.", 
   widget, hour, minute, second, 
   fltk_clock, int, int, int)

{
  GFL_ASSERT_4(fltk_clock_value, widget, hour, minute, second);
  Fl_Clock *wi = (Fl_Clock *) gh_cdr (widget);
  wi->value(gh_scm2int(hour), gh_scm2int(minute), gh_scm2int(second));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_clock_init ()
{
  GFL_NEW_TYPE (fltk_clock);
  GFL_NEW_PROCEDURE (fltk_clock_new);
  GFL_NEW_PROCEDURE (fltk_clock_hour);
  GFL_NEW_PROCEDURE (fltk_clock_minute);
  GFL_NEW_PROCEDURE (fltk_clock_second);
  GFL_NEW_PROCEDURE (fltk_clock_value);
}
