/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Color_Chooser.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Color_Chooser *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Color_Chooser) + sizeof (Fl_Color_Chooser));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Color_Chooser *widget = (Fl_Color_Chooser *) gh_cdr (widget_smob);

  scm_puts ("#<widget Fl_Color_Chooser ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);

  return 1;
}

GFL_SUBCLASS_TYPE ("fltk-color-chooser",
		   "fltk-color-chooser provides a way for a user to select an rgb value.",
		   fltk_color_chooser, fltk_group,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_color_chooser_new,
  "fltk-color-chooser::new",
  "Creates a new fltk-color-chooser with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  color_chooser,
  fltk_color_chooser )
{
  GFL_ASSERT_5 (fltk_color_chooser_new, x, y, w, h, label);

  Fl_Color_Chooser * widget = new Fl_Color_Chooser(gh_scm2int(x), gh_scm2int(y), 
					   gh_scm2int(w), gh_scm2int(h),
					   gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_color_chooser, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
  ( fltk_color_chooser_get_hue,
   "fltk-color-chooser::get-hue",
   "Gets the current hue, where 0 <= hue < 6. Zero is red, one is yellow, "
   "two is green, etc.",
   widget, 
   fltk_color_chooser,
   hue,
   float )
{
  GFL_ASSERT_1 (fltk_color_chooser_get_hue, widget);
  Fl_Color_Chooser *w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_double2scm(w->hue());
}

GFL_PROCEDURE_1_1
  ( fltk_color_chooser_get_saturation,
   "fltk-color-chooser::get-saturation",
   "Gets the current saturation, where 0 <= saturation < 1.",
   widget, 
   fltk_color_chooser,
   saturation,
   float )
{
  GFL_ASSERT_1 (fltk_color_chooser_get_saturation, widget);
  Fl_Color_Chooser *w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_double2scm(w->saturation());
}

GFL_PROCEDURE_1_1
  ( fltk_color_chooser_get_value,
   "fltk-color-chooser::get-value",
   "Gets the current value, where 0 <= value < 1.",
   widget, 
   fltk_color_chooser,
   value,
   float )
{
  GFL_ASSERT_1 (fltk_color_chooser_get_value, widget);
  Fl_Color_Chooser *w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_double2scm(w->value());
}

GFL_PROCEDURE_1_1
  ( fltk_color_chooser_get_r,
   "fltk-color-chooser::get-r",
   "Gets the red component of the selected color, where 0 <= red < 1.",
   widget, 
   fltk_color_chooser,
   red,
   float )
{
  GFL_ASSERT_1 (fltk_color_chooser_get_r, widget);
  Fl_Color_Chooser *w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_double2scm(w->r());
}

GFL_PROCEDURE_1_1
  ( fltk_color_chooser_get_g,
   "fltk-color-chooser::get-g",
   "Gets the green component of the selected color, where 0 <= green < 1.",
   widget, 
   fltk_color_chooser,
   green,
   float )
{
  GFL_ASSERT_1 (fltk_color_chooser_get_g, widget);
  Fl_Color_Chooser *w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_double2scm(w->g());
}

GFL_PROCEDURE_1_1
  ( fltk_color_chooser_get_b,
   "fltk-color-chooser::get-b",
   "Gets the blue component of the selected color, where 0 <= blue < 1.",
   widget, 
   fltk_color_chooser,
   blue,
   float )
{
  GFL_ASSERT_1 (fltk_color_chooser_get_b, widget);
  Fl_Color_Chooser *w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_double2scm(w->b());
}

GFL_PROCEDURE_4_0
  ( fltk_color_chooser_set_rgb,
   "fltk-color-chooser::set-rgb",
   "Sets the current color with the given r-g-b values.",
   widget, r, g, b,
   fltk_color_chooser, float, float, float )
{
  GFL_ASSERT_4 (fltk_color_chooser_set_rgb, widget, r, g, b);
  Fl_Color_Chooser * w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_int2scm(w->rgb(gh_scm2double(r),
				gh_scm2double(g),
				gh_scm2double(b)));
}

GFL_PROCEDURE_4_0
  ( fltk_color_chooser_set_hsv,
   "fltk-color-chooser::set-hsv",
   "Sets the current color with the given h-s-v values.",
   widget, h, s, v,
   fltk_color_chooser, float, float, float )
{
  GFL_ASSERT_4 (fltk_color_chooser_set_hsv, widget, h, s, v);
  Fl_Color_Chooser * w = (Fl_Color_Chooser *) gh_cdr (widget);
  return gh_int2scm(w->hsv(gh_scm2double(h),
			   gh_scm2double(s),
			   gh_scm2double(v)));
}

GFL_PROCEDURE_3_3
  ( fltk_color_chooser_hsv2rgb,
   "fltk-color-chooser::hsv2rgb",
   "Converts h-s-v color components to r-g-b components.",
   h, s, v,
   float, float, float,
   r, g, b,
   float, float, float )
{
  GFL_ASSERT_3 (fltk_color_chooser_hsv2rgb, h, s, v);
  double r, g, b;
  Fl_Color_Chooser::hsv2rgb(gh_scm2double(h), gh_scm2double(s),
			    gh_scm2double(v), r, g, b);
  return gh_cons(gh_double2scm(r), 
		 gh_cons(gh_double2scm(g), 
			 gh_cons(gh_double2scm(b), SCM_EOL)));
}

GFL_PROCEDURE_3_3
  ( fltk_color_chooser_rgb2hsv,
   "fltk-color-chooser::rgb2hsv",
   "Converts r-g-b color components to h-s-v components.",
   r, g, b,
   float, float, float,
   h, s, v,
   float, float, float )
{
  GFL_ASSERT_3 (fltk_color_chooser_rgb2hsv, r, g, b);
  double h, s, v;
  Fl_Color_Chooser::hsv2rgb(gh_scm2double(r), gh_scm2double(g),
			    gh_scm2double(b), h, s, v);
  return gh_cons(gh_double2scm(h), 
		 gh_cons(gh_double2scm(s), 
			 gh_cons(gh_double2scm(v), SCM_EOL)));
}

/*
   4.  Initialization Hook:
*/

void
fltk_color_chooser_init ()
{
  GFL_NEW_TYPE (fltk_color_chooser);
  GFL_NEW_PROCEDURE (fltk_color_chooser_new);
  GFL_NEW_PROCEDURE (fltk_color_chooser_get_hue);
  GFL_NEW_PROCEDURE (fltk_color_chooser_get_saturation);
  GFL_NEW_PROCEDURE (fltk_color_chooser_get_value);
  GFL_NEW_PROCEDURE (fltk_color_chooser_get_r);
  GFL_NEW_PROCEDURE (fltk_color_chooser_get_g);
  GFL_NEW_PROCEDURE (fltk_color_chooser_get_b);
  GFL_NEW_PROCEDURE (fltk_color_chooser_set_rgb);
  GFL_NEW_PROCEDURE (fltk_color_chooser_set_hsv);
  GFL_NEW_PROCEDURE (fltk_color_chooser_rgb2hsv);
  GFL_NEW_PROCEDURE (fltk_color_chooser_hsv2rgb);
}
