/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Counter.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Counter *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Counter) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Counter *widget = (Fl_Counter *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Counter ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-counter",
		   "fltk-counter provides a widget to control a single float value.",
		   fltk_counter, fltk_valuator,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1
( fltk_counter_new,
  "fltk_counter_new ",
  "Creates a new fltk-counter.",
  x, y, w, h, label,
  int, int, int, int, str,
  widget,
  fltk_counter)
{
  GFL_ASSERT_5 (fltk_counter_new, x, y, w, h, label);
  Fl_Counter *widget = new Fl_Counter(gh_scm2int(x), gh_scm2int(y), 
				      gh_scm2int(w), gh_scm2int(h),
				      gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_counter, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_2_0
  ( fltk_counter_set_step, 
   "fltk-counter::set-step", 
   "Sets the step value for single arrow buttons.", 
   widget, a, 
   fltk_counter, double)
{
  GFL_ASSERT_2(fltk_counter_set_step, widget, a);
  Fl_Counter *wi = (Fl_Counter *) gh_cdr (widget);
  wi->step(gh_scm2double(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
  ( fltk_counter_set_lstep, 
   "fltk-counter::set-lstep", 
   "Sets the step value for double arrow buttons.", 
   widget, a, 
   fltk_counter, double)
{
  GFL_ASSERT_2(fltk_counter_set_lstep, widget, a);
  Fl_Counter *wi = (Fl_Counter *) gh_cdr (widget);
  wi->lstep(gh_scm2double(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
  ( fltk_counter_type, 
   "fltk-counter::type", 
   "Sets the counter type.  Valid values include: FL_NORMAL_COUNTER, and FL_SIMPLE_COUNTER.",
   widget, a, 
   fltk_counter, int)
{
  GFL_ASSERT_2(fltk_counter_type, widget, a);
  Fl_Counter *wi = (Fl_Counter *) gh_cdr (widget);
  wi->type((uchar)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_counter_init ()
{
  GFL_NEW_TYPE (fltk_counter);
  GFL_NEW_PROCEDURE (fltk_counter_new);
  GFL_NEW_PROCEDURE (fltk_counter_set_lstep);
  GFL_NEW_PROCEDURE (fltk_counter_set_step);
  GFL_NEW_PROCEDURE (fltk_counter_type);
}
