/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Dial.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Dial *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Dial) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Dial *widget = (Fl_Dial *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Dial ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-dial",
		   "fltk-dial provides a circular dial to control a floating point value.",
		   fltk_dial, fltk_valuator,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1
( fltk_dial_new,
  "fltk-dial::new ",
  "Creates a new fltk-dial with the given position, size and label.",
  x, y, w, h, label,
  int, int, int, int, str,
  widget,
  fltk_dial)
{
  GFL_ASSERT_5 (fltk_dial_new, x, y, w, h, label);
  Fl_Dial *widget = new Fl_Dial(gh_scm2int(x), gh_scm2int(y), 
				gh_scm2int(w), gh_scm2int(h),
				gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_dial, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_3_0
  ( fltk_dial_angles, 
   "fltk-dial::angles", 
   "Sets the angles used for minimum and maximum values.", 
   widget, a, b, 
   fltk_dial, double, double)

{
  GFL_ASSERT_3(fltk_dial_angles, widget, a, b);
  Fl_Dial *wi = (Fl_Dial *) gh_cdr (widget);
  wi->angles(gh_scm2double(a), gh_scm2double(b));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
  ( fltk_dial_type, 
   "fltk-dial::type", 
   "Sets the dial type.  Valid values include: FL_NORMAL_DIAL, FL_LINE_DIAL, and FL_FILL_DIAL.", 
   widget, a, 
   fltk_dial, int)
{
  GFL_ASSERT_2(fltk_dial_type, widget, a);
  Fl_Dial *wi = (Fl_Dial *) gh_cdr (widget);
  wi->type((uchar)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_dial_init ()
{
  GFL_NEW_TYPE (fltk_dial);
  GFL_NEW_PROCEDURE (fltk_dial_new);
  GFL_NEW_PROCEDURE (fltk_dial_angles);
  GFL_NEW_PROCEDURE (fltk_dial_type);
}
