/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Double_Window.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Double_Window *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Double_Window) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Double_Window *widget = (Fl_Double_Window *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Double_Window ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-double-window",
		   "fltk-double-window provides a window with double buffering.",
		   fltk_double_window, fltk_window,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1
  ( fltk_double_window_new,
   "fltk-double-window::new",
   "Creates an fltk-double-window with the given position, size and label.",
   x, y, w, h, label,
   int, int, int, int, str,
   window,
   fltk_double_window )
{
  GFL_ASSERT_5 (fltk_double_window_new, x, y, w, h, label);
  Fl_Double_Window * widget = new Fl_Double_Window(gh_scm2int(x), gh_scm2int(y), 
						   gh_scm2int(w), gh_scm2int(h),
						   gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_double_window, widget_smob, widget);
  return widget_smob;
}

/*
   4.  Initialization Hook:
*/

void
fltk_double_window_init ()
{
  GFL_NEW_TYPE (fltk_double_window);
  GFL_NEW_PROCEDURE (fltk_double_window_new);
}
