/*
  Copyright 1999 by James Dean Palmer.
 
  Thi!s library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/fl_message.H>
#include <FL/fl_show_colormap.H>
#include <FL/Fl_Color_Chooser.H>
#include <FL/fl_file_chooser.H>
#include <FL/filename.H>
#include <gFL/guile_fltk.h>

GFL_EXTERN_TYPE (fltk_widget);

GFL_PROCEDURE_0_0
( fltk_fl_run,
  "fltk-fl::run",
  "Starts the fltk wait-event loop.")
{
  Fl::run();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_0_1
( fltk_fl_event_clicks,
  "fltk-fl::event-clicks",
  "Returns the number of registered mouse clicks.",
  clicks,
  int )
{
  return gh_long2scm(Fl::event_clicks());
}

GFL_PROCEDURE_0_0
( fltk_fl_beep,
  "fltk-fl::beep",
  "Emits a beep signal to the console.")
{
  printf("\007"); 
  fflush(stdout);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_fl_message,
  "fltk-fl::message",
  "Displays the passed string in an \"info\" pop-up box with an \"OK\" button.",
  message,
  str)
{
  GFL_ASSERT_1 (fltk_fl_message, message);
  char *t = gh_scm2newstr (message, NULL);
  fl_message (t);
  free (t);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_fl_alert,
  "fltk-fl::alert",
  "Displays the passed string in an \"alert\" pop-up box with an \"OK\" button.",
  message,
  str)
{
  GFL_ASSERT_1 (fltk_fl_alert, message);
  char *t = gh_scm2newstr (message, NULL);
  fl_alert (t);
  free (t);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_fl_ask,
  "fltk-fl::ask",
  "Displays the passed string in an \"query\" pop-up box with a \"Yes\""
  "button and a \"No\" button.,",
  message,
  str,
  ask,
  bool)
{
  GFL_ASSERT_1 (fltk_fl_ask, message);
  char *t = gh_scm2newstr (message, NULL);
  int r = fl_ask (t);
  free (t);
  if (r) 
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_4_1
( fltk_fl_choice,
  "fltk-fl::choice",
  "Presents a requester with the given label and three choices.",
  label, choice1, choice2, choice3,
  str, str, str, str,
  selected,
  int )
{
  GFL_ASSERT_4 (fltk_fl_choice, label, choice1, choice2, choice3);
  char *t = gh_scm2newstr (label, NULL);
  char *b0 = gh_scm2newstr (choice1, NULL);
  char *b1 = gh_scm2newstr (choice2, NULL);
  char *b2 = gh_scm2newstr (choice3, NULL);
  int selected = fl_choice((const char *)t, (const char *)b0, (const char *)b1, (const char *)b2);
  free (t);
  free (b0);
  free (b1);
  free (b2);
  return gh_int2scm (selected);
}

GFL_PROCEDURE_2_1
( fltk_fl_input,
  "fltk-fl::input",
  "Provides a window with a feild for the user to edit a string of text.",
  title, text,
  str, str,
  new_text,
  str )
{
  GFL_ASSERT_2 (fltk_fl_input, title, text);
  char *t = gh_scm2newstr (title, NULL);
  char *t2 = gh_scm2newstr (text, NULL);
  const char *t3 = fl_input (t, t2);
  free (t);
  free (t2);
  return gh_str02scm((char *)t3);
}

GFL_PROCEDURE_2_1
( fltk_fl_password,
  "fltk-fl::password",
  "Provides a window with a password feild for the user to edit a string of text.",
  title, text,
  str, str,
  new_text,
  str )
{
  GFL_ASSERT_2 (fltk_fl_password, title, text);
  char *t = gh_scm2newstr (title, NULL);
  char *t2 = gh_scm2newstr (text, NULL);
  const char *t3 = fl_input (t, t2);
  free (t);
  free (t2);
  return gh_str02scm((char *)t3);
}

GFL_PROCEDURE_4_4
( fltk_fl_color_chooser,
  "fltk-fl::color-chooser",
  "Pops up a window to let the user select an arbitrary RGB value.  Each "
  "value range is from 0.0 to 1.0.",
  title, red, green, blue,
  str, double, double, double,
  okay, red2, green2, blue2, 
  bool, double, double, double )
{ 
  GFL_ASSERT_4 (fltk_fl_color_chooser, title, red, green, blue);
  double r, g, b;
  SCM okay = SCM_BOOL_F;
  char *t = gh_scm2newstr(title, NULL);
  r = gh_scm2double (red);
  g = gh_scm2double (green);
  b = gh_scm2double (blue);
  if (fl_color_chooser (t, r, g, b))
    okay = SCM_BOOL_T;
  free (t);
  return gh_cons (okay, 
		  gh_cons (gh_double2scm(r), 
			   gh_cons (gh_double2scm(g),
				    gh_cons (gh_double2scm(b), SCM_EOL))));
}

GFL_PROCEDURE_1_1
( fltk_fl_show_colormap,
  "fltk-fl::show-colormap",
  "Pops up a list of 256 colors and lets the user select one of these colors.",
  old_color,
  int,
  new_color,
  int )
{
  GFL_ASSERT_1 (fltk_fl_show_colormap, old_color);
  return gh_int2scm ((int)fl_show_colormap ((Fl_Color)gh_scm2int(old_color)));
}

GFL_PROCEDURE_2_0
( fltk_fl_set_message_font, 
  "fltk-fl::set-message-font", 
  "Sets the message font and font size..", 
  fontid, size, 
  int, int)
{
  GFL_ASSERT_2 (fltk_fl_set_message_font, fontid, size);
  fl_message_font((Fl_Font)fontid, (uchar)size);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_0_1
( fltk_fl_get_message_icon, 
  "fltk-fl::get-message-icon", 
  "Gets box at the left edge of all the popups.",
  widget,
  fltk_widget )
{
  return gfl_lookup (fl_message_icon());
}

GFL_PROCEDURE_3_1
( fltk_fl_file_chooser, 
  "fltk-fl::file-chooser", 
  "A file chooser that makes it easy to choose files from large directories.", 
  message, pattern, name,
  str, str, str,
  result,
  str )
{
  GFL_ASSERT_3 (fltk_fl_file_chooser, message, pattern, name);
  char *m = gh_scm2newstr (message, NULL);
  char *p = gh_scm2newstr (pattern, NULL);
  char *n = gh_scm2newstr (name, NULL);
  char *x = fl_file_chooser((const char *)m, (const char *)p, (const char *)n);
  return (gh_str02scm (x));
}

GFL_PROCEDURE_1_0
( fltk_fl_file_chooser_callback,
  "fltk-fl::file-chooser-callback",
  "Sets a function that is called every time the user clicks a file "
  "in the currently popped-up file chooser.",
  function,
  callback )
{
  GFL_ASSERT_1 (fltk_fl_file_chooser_callback, function);
  // FIXME:
  // fl_file_chooser_callback(a);
}

GFL_PROCEDURE_1_1
( fltk_fl_filename_list,
  "fltk-fl::filename-list",
  "Returns the file listing for the specified directory.",
  directory,
  str,
  rvalue,
  int )
{
  GFL_ASSERT_1 (fltk_fl_filename_list, directory);
  int i;
  char *d = gh_scm2newstr (directory, NULL);
  dirent ***list = 0;
  int return_value = filename_list (d, list);
  if (return_value == 0)
    return SCM_BOOL_F;
  for (i = return_value; i > 0; i--)
    {
      // FIXME: cons these values together in a sane way, or
      // use a vector..
    }
  for (i = return_value; i > 0;) free((void*)(list[--i]));
  free((void*)list);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_fl_filename_is_dir,
  "fltk-fl::filename-is-dir",
  "Returns true if the file is a directory.",
  directory,
  str,
  rvalue,
  bool )
{
  GFL_ASSERT_1 (fltk_fl_filename_is_dir, directory);
  char *d = gh_scm2newstr (directory, NULL);
  if (filename_isdir (d))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
( fltk_fl_filename_name,
  "fltk-fl::filename-name",
  "Returns the filename from the file and it's path.",
  complete_filename,
  str,
  filename,
  str)
{
  GFL_ASSERT_1 (fltk_fl_filename_name, complete_filename);
  char *d = gh_scm2newstr (complete_filename, NULL);
  const char *d2 = filename_name (d);
  if (!d2)
    return SCM_BOOL_F;
  SCM rvalue = gh_str02scm ((char *)d2);
  free (d);
  return rvalue;
}

GFL_PROCEDURE_1_1
( fltk_fl_filename_ext,
  "fltk-fl::filename-ext",
  "Returns the extension from the file and it's path.",
  complete_filename,
  str,
  extension,
  str)
{
  GFL_ASSERT_1 (fltk_fl_filename_ext, complete_filename);
  char *d = gh_scm2newstr (complete_filename, NULL);
  char *d2 = (char *)filename_ext (d);
  if (!d2)
    return SCM_BOOL_F;
  SCM rvalue = gh_str02scm (d2);
  return rvalue;
}

GFL_PROCEDURE_2_1
( fltk_fl_filename_setext,
  "fltk-fl::filename-setext",
  "Replaces the extension in the filename with ext.",
  filename, ext,
  str, str,
  new_filename,
  str)
{
  GFL_ASSERT_2 (fltk_fl_filename_setext, filename, ext);
  // FIXME: This needs to be written in a safer way than the Fltk way.
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
( fltk_fl_filename_expand,
  "fltk-fl::filename-expand",
  "Expands filename with environmental variables.",
  in,
  str,
  out,
  str)
{
  GFL_ASSERT_1 (fltk_fl_filename_expand, in);
  char b[1024];
  char *a = gh_scm2newstr (in, NULL);
  filename_expand (b, (const char *)a);
  if (!b)
    return SCM_BOOL_F;
  free (a);
  return gh_str02scm (b);
}

GFL_PROCEDURE_1_1
( fltk_fl_filename_absolute,
  "fltk-fl::filename-absolute",
  "Converts ./, ../, and the current working directory to an absolute file name.",
  in,
  str,
  out,
  str)
{
  GFL_ASSERT_1 (fltk_fl_filename_absolute, in);
  char b[1024];
  char *a = gh_scm2newstr (in, NULL);
  filename_absolute (b, (const char *)a);
  if (!b)
    return SCM_BOOL_F;
  free (a);
  return gh_str02scm (b);
}

GFL_PROCEDURE_2_1
( fltk_fl_filename_match,
  "fltk-fl::filename-match",
  "Returns true if the filename matches a pattern.",
  filename, pattern,
  str, str,
  out,
  str)
{
  GFL_ASSERT_2 (fltk_fl_filename_match, filename, pattern);
  SCM rvalue;
  char *f = gh_scm2newstr (filename, NULL);
  char *p = gh_scm2newstr (pattern, NULL);
  if (filename_match ((const char *)f, (const char *)p))
    rvalue = SCM_BOOL_T;
  else
    rvalue = SCM_BOOL_F;
  free (f);
  free (p);
  return rvalue;
}

GFL_PROCEDURE_3_0
( fltk_fl_background,
  "fltk-fl::background",
  "Changes the background color most widgets use.",
  red, green, blue,
  int, int, int )
{
  Fl::background (gh_scm2int(red), gh_scm2int(green), gh_scm2int(blue));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_fl_background2,
  "fltk-fl::background2",
  "Changes the input background color most widgets use.",
  red, green, blue,
  int, int, int )
{
  GFL_ASSERT_3 (fltk_fl_background2, red, green, blue);
  Fl::background (gh_scm2int(red), gh_scm2int(green), gh_scm2int(blue));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_0_1
( fltk_fl_get_below_mouse,
  "fltk-fl::get-below-mouse",
  "Get the widget below the mouse.",
  widget,
  fltk_widget )
{
  Fl_Widget *w = Fl::belowmouse();
  if (w)
    return gfl_lookup(w);
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_0
( fltk_fl_set_below_mouse,
  "fltk-fl::get-below-mouse",
  "Set the widget below the mouse.",
  widget,
  fltk_widget )
{
  GFL_ASSERT_1 (fltk_fl_set_below_mouse, widget);
  Fl_Widget *w = (Fl_Widget *) gh_cdr(widget);
  if (w)
    Fl::belowmouse(w);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_fl_box_dh,
  "fltk-fl::box-dh",
  "Get the height offset for the given boxtype.",
  boxtype,
  int,
  dh,
  int )
{
  GFL_ASSERT_1 (fltk_fl_box_dh, boxtype);
  return (gh_int2scm (Fl::box_dh ((Fl_Boxtype)gh_scm2int (boxtype))));
}

GFL_PROCEDURE_1_1
( fltk_fl_box_dw,
  "fltk-fl::box-dw",
  "Get the width offset for the given boxtype.",
  boxtype,
  int,
  dw,
  int )
{
  GFL_ASSERT_1 (fltk_fl_box_dw, boxtype);
  return (gh_int2scm (Fl::box_dw ((Fl_Boxtype)gh_scm2int (boxtype))));
}

GFL_PROCEDURE_1_1
( fltk_fl_box_dx,
  "fltk-fl::box-dx",
  "Get the x offset for the given boxtype.",
  boxtype,
  int,
  dx,
  int )
{
  GFL_ASSERT_1 (fltk_fl_box_dx, boxtype);
  return gh_int2scm (Fl::box_dx ((Fl_Boxtype)gh_scm2int (boxtype)));
}

GFL_PROCEDURE_1_1
( fltk_fl_box_dy,
  "fltk-fl::box-dy",
  "Get the y offset for the given boxtype.",
  boxtype,
  int,
  dy,
  int )
{
  GFL_ASSERT_1 (fltk_fl_box_dy, boxtype);
  return gh_int2scm (Fl::box_dy ((Fl_Boxtype)gh_scm2int (boxtype)));
}

GFL_PROCEDURE_0_1
( fltk_fl_check,
  "fltk-fl::check",
  "Does an fltk-fl-wait 0, but it does not return the elapsed time and thus may be faster.",
  windows_displayed,
  bool )
{
  if (Fl::check ())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_0_1
( fltk_fl_damage,
  "fltk-fl::damage",
  "If true, flush() will do something.",
  windows_displayed,
  bool )
{
  if (Fl::damage ())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_0
( fltk_fl_display,
  "fltk-fl::display",
  "Sets the X display to open X windows on.",
  display,
  str )
{
  char *c = gh_scm2newstr (display, NULL);
  Fl::display (c);
  free (c);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_0_0
( fltk_fl_enable_symbols,
  "fltk-fl::enable-symbols",
  "Enables symbol drawing code." )
{
  Fl::enable_symbols ();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_0_1
( fltk_fl_event_button,
  "fltk-fl::event-button",
  "Returns which mouse button was pressed.",
   button,
   int)
{
  return gh_int2scm (Fl::event_button ());
}

GFL_PROCEDURE_0_1
( fltk_fl_get_event_clicks,
  "fltk-fl::get-event-clicks",
  "Returns N-1 for N clicks.",
   button,
   int)
{
  return gh_int2scm (Fl::event_clicks ());
}

GFL_PROCEDURE_1_0
( fltk_fl_set_event_clicks,
  "fltk-fl::set-event-clicks",
  "Directly sets the number of event clicks.",
   clicks,
   int)
{
  GFL_ASSERT_1 (fltk_fl_set_event_clicks, clicks);
  Fl::event_clicks (gh_scm2int (clicks));
  return SCM_UNSPECIFIED;
}

void
fltk_fl_init ()
{
  GFL_NEW_PROCEDURE (fltk_fl_run);
  GFL_NEW_PROCEDURE (fltk_fl_event_clicks);
  GFL_NEW_PROCEDURE (fltk_fl_beep);
  GFL_NEW_PROCEDURE (fltk_fl_message);
  GFL_NEW_PROCEDURE (fltk_fl_alert);
  GFL_NEW_PROCEDURE (fltk_fl_ask);
  GFL_NEW_PROCEDURE (fltk_fl_choice);
  GFL_NEW_PROCEDURE (fltk_fl_input);
  GFL_NEW_PROCEDURE (fltk_fl_password);
  GFL_NEW_PROCEDURE (fltk_fl_color_chooser);
  GFL_NEW_PROCEDURE (fltk_fl_show_colormap);
  GFL_NEW_PROCEDURE (fltk_fl_set_message_font);
  GFL_NEW_PROCEDURE (fltk_fl_get_message_icon);
  GFL_NEW_PROCEDURE (fltk_fl_file_chooser);
  GFL_NEW_PROCEDURE (fltk_fl_file_chooser_callback);
  GFL_NEW_PROCEDURE (fltk_fl_filename_list);
  GFL_NEW_PROCEDURE (fltk_fl_filename_is_dir);
  GFL_NEW_PROCEDURE (fltk_fl_filename_name);
  GFL_NEW_PROCEDURE (fltk_fl_filename_ext);
  GFL_NEW_PROCEDURE (fltk_fl_filename_setext);
  GFL_NEW_PROCEDURE (fltk_fl_filename_expand);
  GFL_NEW_PROCEDURE (fltk_fl_filename_match);

  gh_define ("FL_WHEN_NEVER", gh_long2scm(FL_WHEN_NEVER));
  gh_define ("FL_WHEN_CHANGED", gh_long2scm(FL_WHEN_CHANGED));
  gh_define ("FL_WHEN_RELEASE", gh_long2scm(FL_WHEN_RELEASE));
  gh_define ("FL_WHEN_RELEASE_ALWAYS", gh_long2scm(FL_WHEN_RELEASE_ALWAYS));
  gh_define ("FL_WHEN_ENTER_KEY", gh_long2scm(FL_WHEN_ENTER_KEY));
  gh_define ("FL_WHEN_ENTER_KEY_ALWAYS", gh_long2scm(FL_WHEN_ENTER_KEY_ALWAYS));
  gh_define ("FL_WHEN_NOT_CHANGED", gh_long2scm(FL_WHEN_NOT_CHANGED));

  gh_define ("FL_SHIFT", gh_long2scm(FL_SHIFT));
  gh_define ("FL_CAPS_LOCK", gh_long2scm(FL_CAPS_LOCK));
  gh_define ("FL_CTRL", gh_long2scm(FL_CTRL));
  gh_define ("FL_ALT", gh_long2scm(FL_ALT));
  gh_define ("FL_NUM_LOCK", gh_long2scm(FL_NUM_LOCK));
  gh_define ("FL_META", gh_long2scm(FL_META));
  gh_define ("FL_SCROLL_LOCK", gh_long2scm(FL_SCROLL_LOCK));
  gh_define ("FL_BUTTON1", gh_long2scm(FL_BUTTON1));
  gh_define ("FL_BUTTON2", gh_long2scm(FL_BUTTON2));
  gh_define ("FL_BUTTON3", gh_long2scm(FL_BUTTON3));

  gh_define ("FL_Button", gh_long2scm(FL_Button));
  gh_define ("FL_BackSpace", gh_long2scm(FL_BackSpace));
  gh_define ("FL_Tab", gh_long2scm(FL_Tab));
  gh_define ("FL_Enter", gh_long2scm(FL_Enter));
  gh_define ("FL_Pause", gh_long2scm(FL_Pause));
  gh_define ("FL_Scroll-Lock", gh_long2scm(FL_Scroll_Lock));
  gh_define ("FL_Escape", gh_long2scm(FL_Escape));
  gh_define ("FL_Home", gh_long2scm(FL_Home));
  gh_define ("FL_Left", gh_long2scm(FL_Left));
  gh_define ("FL_Up", gh_long2scm(FL_Up));
  gh_define ("FL_Right", gh_long2scm(FL_Right));
  gh_define ("FL_Down", gh_long2scm(FL_Down));
  gh_define ("FL_Page_Up", gh_long2scm(FL_Page_Up));
  gh_define ("FL_Page_Down", gh_long2scm(FL_Page_Down));
  gh_define ("FL_End", gh_long2scm(FL_End));
  gh_define ("FL_Print", gh_long2scm(FL_Print));
  gh_define ("FL_Insert", gh_long2scm(FL_Insert));
  gh_define ("FL_Menu", gh_long2scm(FL_Menu));
  gh_define ("FL_Num_Lock", gh_long2scm(FL_Num_Lock));
  gh_define ("FL_KP", gh_long2scm(FL_KP));
  gh_define ("FL_KP_Enter", gh_long2scm(FL_KP_Enter));
  gh_define ("FL_KP_Last", gh_long2scm(FL_KP_Last));

  // Enumerations:
  gh_define ("FL_MAJOR_VERSION", gh_long2scm(FL_MAJOR_VERSION));
  gh_define ("FL_MINOR_VERSION", gh_long2scm(FL_MINOR_VERSION));
  gh_define ("FL_PATCH_VERSION", gh_long2scm(FL_PATCH_VERSION));
  gh_define ("FL_NO_EVENT", gh_long2scm(FL_NO_EVENT));
  gh_define ("FL_PUSH", gh_long2scm(FL_PUSH));
  gh_define ("FL_RELEASE", gh_long2scm(FL_RELEASE));
  gh_define ("FL_ENTER", gh_long2scm(FL_ENTER));
  gh_define ("FL_LEAVE", gh_long2scm(FL_LEAVE));
  gh_define ("FL_DRAG", gh_long2scm(FL_DRAG));
  gh_define ("FL_FOCUS", gh_long2scm(FL_FOCUS));
  gh_define ("FL_UNFOCUS", gh_long2scm(FL_UNFOCUS));
  gh_define ("FL_KEYBOARD", gh_long2scm(FL_KEYBOARD));
  gh_define ("FL_CLOSE", gh_long2scm(FL_CLOSE));
  gh_define ("FL_MOVE", gh_long2scm(FL_MOVE));
  gh_define ("FL_SHORTCUT", gh_long2scm(FL_SHORTCUT));
  gh_define ("FL_DEACTIVATE", gh_long2scm(FL_DEACTIVATE));
  gh_define ("FL_ACTIVATE", gh_long2scm(FL_ACTIVATE));
  gh_define ("FL_HIDE", gh_long2scm(FL_HIDE));
  gh_define ("FL_SHOW", gh_long2scm(FL_SHOW));
  gh_define ("FL_PASTE", gh_long2scm(FL_PASTE));
  gh_define ("FL_SELECTIONCLEAR", gh_long2scm(FL_SELECTIONCLEAR));
  
  gh_define ("FL_NO_BOX", gh_long2scm(FL_NO_BOX));
  gh_define ("FL_FLAT_BOX", gh_long2scm(FL_FLAT_BOX));
  gh_define ("FL_UP_BOX", gh_long2scm(FL_UP_BOX)); 
  gh_define ("FL_DOWN_BOX", gh_long2scm(FL_DOWN_BOX)); 
  gh_define ("FL_UP_FRAME", gh_long2scm(FL_UP_FRAME)); 
  gh_define ("FL_DOWN_FRAME", gh_long2scm(FL_DOWN_FRAME)); 
  
  gh_define ("FL_THIN_UP_BOX", gh_long2scm(FL_THIN_UP_BOX)); 
  gh_define ("FL_THIN_DOWN_BOX", gh_long2scm(FL_THIN_DOWN_BOX)); 
  gh_define ("FL_THIN_UP_FRAME", gh_long2scm(FL_THIN_UP_FRAME)); 
  gh_define ("FL_THIN_DOWN_FRAME", gh_long2scm(FL_THIN_DOWN_FRAME)); 
  
  gh_define ("FL_ENGRAVED_BOX", gh_long2scm(FL_ENGRAVED_BOX)); 
  gh_define ("FL_EMBOSSED_BOX", gh_long2scm(FL_EMBOSSED_BOX)); 
  gh_define ("FL_ENGRAVED_FRAME", gh_long2scm(FL_ENGRAVED_FRAME)); 
  gh_define ("FL_EMBOSSED_FRAME", gh_long2scm(FL_EMBOSSED_FRAME)); 
  gh_define ("FL_BORDER_BOX", gh_long2scm(FL_BORDER_BOX)); 
  gh_define ("FL_BORDER_FRAME", gh_long2scm(FL_BORDER_FRAME)); 
  gh_define ("FL_SHADOW_BOX", gh_long2scm(FL_SHADOW_BOX)); 
  gh_define ("FL_SHADOW_FRAME", gh_long2scm(FL_SHADOW_FRAME)); 
  gh_define ("FL_ROUNDED_BOX", gh_long2scm(FL_ROUNDED_BOX)); 
  gh_define ("FL_ROUNDED_FRAME", gh_long2scm(FL_ROUNDED_FRAME)); 
  gh_define ("FL_RSHADOW_BOX", gh_long2scm(FL_RSHADOW_BOX)); 
  gh_define ("FL_RFLAT_BOX", gh_long2scm(FL_RFLAT_BOX)); 
  gh_define ("FL_ROUND_DOWN_BOX", gh_long2scm(FL_ROUND_DOWN_BOX));    
  gh_define ("FL_ROUND_UP_BOX", gh_long2scm(FL_ROUND_UP_BOX));    
  gh_define ("FL_DIAMOND_DOWN_BOX", gh_long2scm(FL_DIAMOND_DOWN_BOX));    
  gh_define ("FL_DIAMOND_UP_BOX", gh_long2scm(FL_DIAMOND_UP_BOX));    
  gh_define ("FL_OVAL_BOX", gh_long2scm(FL_OVAL_BOX)); 
  gh_define ("FL_OVAL_FRAME", gh_long2scm(FL_OVAL_FRAME)); 
  gh_define ("FL_OSHADOW_BOX", gh_long2scm(FL_OSHADOW_BOX)); 
  gh_define ("FL_OFLAT_BOX", gh_long2scm(FL_OFLAT_BOX)); 
  gh_define ("FL_FREE_BOXTYPE", gh_long2scm(FL_FREE_BOXTYPE)); 
   
  gh_define ("FL_NORMAL_LABEL", gh_long2scm(FL_NORMAL_LABEL)); 
  gh_define ("FL_NO_LABEL", gh_long2scm(FL_NO_LABEL)); 
  gh_define ("FL_SYMBOL_LABEL", gh_long2scm(FL_SYMBOL_LABEL)); 
  gh_define ("FL_SHADOW_LABEL", gh_long2scm(FL_SHADOW_LABEL)); 
  gh_define ("FL_ENGRAVED_LABEL", gh_long2scm(FL_ENGRAVED_LABEL)); 
  gh_define ("FL_EMBOSSED_LABEL", gh_long2scm(FL_EMBOSSED_LABEL)); 
  gh_define ("FL_BITMAP_LABEL", gh_long2scm(_FL_BITMAP_LABEL)); 
  gh_define ("FL_PIXMAP_LABEL", gh_long2scm(_FL_PIXMAP_LABEL)); 
  gh_define ("FL_IMAGE_LABEL", gh_long2scm(_FL_IMAGE_LABEL)); 
  gh_define ("FL_MULTI_LABEL", gh_long2scm(_FL_MULTI_LABEL)); 
  gh_define ("FL_FREE_LABELTYPE", gh_long2scm(FL_FREE_LABELTYPE)); 
   
  gh_define("FL_ALIGN_CENTER", gh_long2scm(FL_ALIGN_CENTER));
  gh_define("FL_ALIGN_TOP", gh_long2scm(FL_ALIGN_TOP));
  gh_define("FL_ALIGN_BOTTOM", gh_long2scm(FL_ALIGN_BOTTOM));
  gh_define("FL_ALIGN_LEFT", gh_long2scm(FL_ALIGN_LEFT));
  gh_define("FL_ALIGN_RIGHT", gh_long2scm(FL_ALIGN_RIGHT));
  gh_define("FL_ALIGN_INSIDE", gh_long2scm(FL_ALIGN_INSIDE));
  gh_define("FL_ALIGN_CLIP", gh_long2scm(FL_ALIGN_CLIP));
  gh_define("FL_ALIGN_WRAP", gh_long2scm(FL_ALIGN_WRAP));
  gh_define("FL_ALIGN_TOP_LEFT", gh_long2scm(FL_ALIGN_TOP_LEFT));
  gh_define("FL_ALIGN_TOP_RIGHT", gh_long2scm(FL_ALIGN_TOP_RIGHT));
  gh_define("FL_ALIGN_BOTTOM_LEFT", gh_long2scm(FL_ALIGN_BOTTOM_LEFT));
  gh_define("FL_ALIGN_BOTTOM_RIGHT", gh_long2scm(FL_ALIGN_BOTTOM_RIGHT));
  gh_define("FL_ALIGN_LEFT_TOP", gh_long2scm(FL_ALIGN_LEFT_TOP));
  gh_define("FL_ALIGN_RIGHT_TOP", gh_long2scm(FL_ALIGN_RIGHT_TOP));
  gh_define("FL_ALIGN_LEFT_BOTTOM", gh_long2scm(FL_ALIGN_LEFT_BOTTOM));
  gh_define("FL_ALIGN_RIGHT_BOTTOM", gh_long2scm(FL_ALIGN_RIGHT_BOTTOM));
  gh_define("FL_ALIGN_NOWRAP", gh_long2scm(FL_ALIGN_NOWRAP));
  
  gh_define("FL_HELVETICA", gh_long2scm(FL_HELVETICA));
  gh_define("FL_HELVETICA_BOLD", gh_long2scm(FL_HELVETICA_BOLD));
  gh_define("FL_HELVETICA_ITALIC", gh_long2scm(FL_HELVETICA_ITALIC));
  gh_define("FL_HELVETICA_BOLD_ITALIC", gh_long2scm(FL_HELVETICA_BOLD_ITALIC));
  gh_define("FL_COURIER", gh_long2scm(FL_COURIER));
  gh_define("FL_COURIER_BOLD", gh_long2scm(FL_COURIER_BOLD));
  gh_define("FL_COURIER_ITALIC", gh_long2scm(FL_COURIER_ITALIC));
  gh_define("FL_COURIER_BOLD_ITALIC", gh_long2scm(FL_COURIER_BOLD_ITALIC));
  gh_define("FL_TIMES", gh_long2scm(FL_TIMES));
  gh_define("FL_TIMES_BOLD", gh_long2scm(FL_TIMES_BOLD));
  gh_define("FL_TIMES_ITALIC", gh_long2scm(FL_TIMES_ITALIC));
  gh_define("FL_TIMES_BOLD_ITALIC", gh_long2scm(FL_TIMES_BOLD_ITALIC));
  gh_define("FL_SYMBOL", gh_long2scm(FL_SYMBOL));
  gh_define("FL_SCREEN", gh_long2scm(FL_SCREEN));
  gh_define("FL_SCREEN_BOLD", gh_long2scm(FL_SCREEN_BOLD));
  gh_define("FL_ZAPF_DINGBATS", gh_long2scm(FL_ZAPF_DINGBATS));
  gh_define("FL_FREE_FONT", gh_long2scm(FL_FREE_FONT));
  gh_define("FL_BOLD", gh_long2scm(FL_BOLD));
  gh_define("FL_ITALIC", gh_long2scm(FL_ITALIC));
  
  gh_define("FL_BLACK", gh_long2scm(FL_BLACK));
  gh_define("FL_RED", gh_long2scm(FL_RED));
  gh_define("FL_GREEN", gh_long2scm(FL_GREEN));
  gh_define("FL_YELLOW", gh_long2scm(FL_YELLOW));
  gh_define("FL_BLUE", gh_long2scm(FL_BLUE));
  gh_define("FL_MAGENTA", gh_long2scm(FL_MAGENTA));
  gh_define("FL_CYAN", gh_long2scm(FL_CYAN));
  gh_define("FL_WHITE", gh_long2scm(FL_WHITE));
  gh_define("FL_INACTIVE_COLOR", gh_long2scm(FL_INACTIVE_COLOR));
  gh_define("FL_SELECTION_COLOR", gh_long2scm(FL_SELECTION_COLOR));
  gh_define("FL_FREE_COLOR", gh_long2scm(FL_FREE_COLOR));
  gh_define("FL_NUM_FREE_COLOR", gh_long2scm(FL_NUM_FREE_COLOR));
  gh_define("FL_GRAY_RAMP", gh_long2scm(FL_GRAY_RAMP));
  gh_define("FL_GRAY0", gh_long2scm(FL_GRAY0));
  gh_define("FL_DARK3", gh_long2scm(FL_DARK3));
  gh_define("FL_DARK2", gh_long2scm(FL_DARK2));
  gh_define("FL_DARK1", gh_long2scm(FL_DARK1));
  gh_define("FL_GRAY", gh_long2scm(FL_GRAY));
  gh_define("FL_LIGHT1", gh_long2scm(FL_LIGHT1));
  gh_define("FL_LIGHT2", gh_long2scm(FL_LIGHT2));
  gh_define("FL_LIGHT3", gh_long2scm(FL_LIGHT3));
  gh_define("FL_COLOR_CUBE", gh_long2scm(FL_COLOR_CUBE));
  
  gh_define("FL_CURSOR_DEFAULT", gh_long2scm(FL_CURSOR_DEFAULT));
  gh_define("FL_CURSOR_ARROW", gh_long2scm(FL_CURSOR_ARROW));
  gh_define("FL_CURSOR_CROSS", gh_long2scm(FL_CURSOR_CROSS));
  gh_define("FL_CURSOR_WAIT", gh_long2scm(FL_CURSOR_WAIT));
  gh_define("FL_CURSOR_INSERT", gh_long2scm(FL_CURSOR_INSERT));
  gh_define("FL_CURSOR_HAND", gh_long2scm(FL_CURSOR_HAND));
  gh_define("FL_CURSOR_HELP", gh_long2scm(FL_CURSOR_HELP));
  gh_define("FL_CURSOR_MOVE", gh_long2scm(FL_CURSOR_MOVE));
  gh_define("FL_CURSOR_NS", gh_long2scm(FL_CURSOR_NS));
  gh_define("FL_CURSOR_WE", gh_long2scm(FL_CURSOR_WE));
  gh_define("FL_CURSOR_NWSE", gh_long2scm(FL_CURSOR_NWSE));
  gh_define("FL_CURSOR_NESW", gh_long2scm(FL_CURSOR_NESW));
  gh_define("FL_CURSOR_NONE", gh_long2scm(FL_CURSOR_NONE));
  gh_define("FL_CURSOR_N", gh_long2scm(FL_CURSOR_N));
  gh_define("FL_CURSOR_NE", gh_long2scm(FL_CURSOR_NE));
  gh_define("FL_CURSOR_E", gh_long2scm(FL_CURSOR_E));
  gh_define("FL_CURSOR_SE", gh_long2scm(FL_CURSOR_SE));
  gh_define("FL_CURSOR_S", gh_long2scm(FL_CURSOR_S));
  gh_define("FL_CURSOR_SW", gh_long2scm(FL_CURSOR_SW));
  gh_define("FL_CURSOR_W", gh_long2scm(FL_CURSOR_W));
  gh_define("FL_CURSOR_NW", gh_long2scm(FL_CURSOR_NW));
  
  gh_define("FL_READ", gh_long2scm(FL_READ));
  gh_define("FL_WRITE", gh_long2scm(FL_WRITE));
  gh_define("FL_EXCEPT", gh_long2scm(FL_EXCEPT));
  gh_define("FL_RGB", gh_long2scm(FL_RGB));
  gh_define("FL_INDEX", gh_long2scm(FL_INDEX));
  gh_define("FL_SINGLE", gh_long2scm(FL_SINGLE));
  gh_define("FL_DOUBLE", gh_long2scm(FL_DOUBLE));
  gh_define("FL_ACCUM", gh_long2scm(FL_ACCUM));
  gh_define("FL_ALPHA", gh_long2scm(FL_ALPHA));
  gh_define("FL_DEPTH", gh_long2scm(FL_DEPTH));
  gh_define("FL_STENCIL", gh_long2scm(FL_STENCIL));
  gh_define("FL_RGB8", gh_long2scm(FL_RGB8));
  gh_define("FL_MULTISAMPLE", gh_long2scm(FL_MULTISAMPLE));
}
