/*
  Copyright 1999 by James Dean Palmer.
  
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
  
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.
  
  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Hold_Browser.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Hold_Browser *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Hold_Browser) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Hold_Browser *widget = (Fl_Hold_Browser *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Hold_Browser ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-hold-browser", 
		   "fltk-hold-browser is a subclass of fltk-browser that lets a single item be selected.",
		   fltk_hold_browser, fltk_browser, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_hold_browser_new,
  "fltk-hold-browser::new",
  "Creates a new fltk-hold-browser with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  hold_browser,
  fltk_hold_browser )
{
  GFL_ASSERT_5 (fltk_hold_browser_new, x, y, w, h, label);
  Fl_Hold_Browser * widget = new Fl_Hold_Browser(gh_scm2int(x), gh_scm2int(y), 
						 gh_scm2int(w), gh_scm2int(h),
						 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_hold_browser, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
( fltk_hold_browser_deselect, 
  "fltk-hold-browser::deselect", 
  "Unselects any selected item.", 
  widget, 
  fltk_hold_browser, 
  rval, 
  bool)
{
  GFL_ASSERT_1(fltk_hold_browser_deselect, widget);  
  Fl_Hold_Browser *wi = (Fl_Hold_Browser *) gh_cdr (widget);
  if (wi->deselect())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
( fltk_hold_browser_select, 
  "fltk-hold-browser::select", 
  "Selects the Nth entry.  Multiple selections are undefined.", 
  widget, N, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_hold_browser_select, widget, N);
  Fl_Browser *wi = (Fl_Hold_Browser *) gh_cdr (widget);
  wi->select(gh_int2scm(N));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_1
( fltk_hold_browser_selected, 
  "fltk-hold-browser::selected", 
  "Returns wether the Nth entry is selected or not.", 
  widget, N, 
  fltk_browser, int, 
  rval, 
  bool)
{
  GFL_ASSERT_2(fltk_hold_browser_selected, widget, N);  
  Fl_Browser *wi = (Fl_Hold_Browser *) gh_cdr (widget);
  if (wi->selected(gh_int2scm(N)))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
( fltk_hold_browser_set_value, 
  "fltk-hold-browser::set-value", 
  "Sets the value (the selected item) to N.",
  widget, N, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_hold_browser_set_value, widget, N);
  Fl_Browser *wi = (Fl_Hold_Browser *) gh_cdr (widget);
  wi->value(gh_int2scm(N));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_hold_browser_get_value, 
  "fltk-hold-browser::get-value", 
  "Gets the value (the selected item).", 
  widget, 
 fltk_hold_browser )
{
  GFL_ASSERT_1(fltk_hold_browser_get_value, widget);
  Fl_Browser *wi = (Fl_Hold_Browser *) gh_cdr (widget);
  return SCM_MAKINUM(wi->value());
}

void
fltk_hold_browser_init ()
{
  GFL_NEW_TYPE (fltk_hold_browser);
  GFL_NEW_PROCEDURE (fltk_hold_browser_new);
  GFL_NEW_PROCEDURE (fltk_hold_browser_deselect);
  GFL_NEW_PROCEDURE (fltk_hold_browser_select);
  GFL_NEW_PROCEDURE (fltk_hold_browser_selected);
  GFL_NEW_PROCEDURE (fltk_hold_browser_set_value);
  GFL_NEW_PROCEDURE (fltk_hold_browser_get_value);
}
