/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Input.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Input *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Input) + sizeof (Fl_Input));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Input *widget = (Fl_Input *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Input ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-input",
		   "fltk-input provides a text input widget.",
		   fltk_input, fltk_input_base,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_input_new,
   "fltk-input::new",
   "Creates a new fltk-input with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   input,
   fltk_input )
{
  GFL_ASSERT_5 (fltk_input_new, x, y, w, h, label);

  Fl_Input * widget = new Fl_Input(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_input, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_2_0
  ( fltk_input_set_cursor_color,
   "fltk-input::set-cursor-color",
   "Sets the color of the cursor.",
   widget, color,
   fltk_input, fltk_color )
{
  GFL_ASSERT_2 (fltk_input_set_cursor_color, widget, color);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  w->cursor_color((Fl_Boxtype)gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_input_get_cursor_color,
   "fltk-input::get-cursor-color",
   "Gets the color of the cursor.",
   widget,
   fltk_input, 
   color,
   fltk_color )
{
  GFL_ASSERT_1 (fltk_input_get_cursor_color, widget);
  Fl_Input * w = (Fl_Input *) gh_cdr (widget);
  return gh_int2scm((int)w->cursor_color());
}

GFL_PROCEDURE_2_1
  ( fltk_input_index,
   "fltk-input::index",
   "Gets the Nth character in the value string.",
   widget, N,
   fltk_input, int,
   color,
   fltk_color )
{
  GFL_ASSERT_2 (fltk_input_index, widget, N);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  if (N < w->size())
    return gh_char2scm(w->index(gh_scm2int(N)));
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
  ( fltk_input_size,
   "fltk-input::size",
   "Returns the number of characters in the value string.",
   widget, 
   fltk_input,
   N,
   int )
{
  GFL_ASSERT_1 (fltk_input_size, widget);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  return gh_int2scm(w->size());
}

GFL_PROCEDURE_1_1
  ( fltk_input_get_value,
   "fltk-input::get-value",
   "Gets the current value.",
   widget, 
   fltk_input,
   value,
   str )
{
  GFL_ASSERT_1 (fltk_input_get_value, widget);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  return gh_str02scm((char *)w->value());
}

GFL_PROCEDURE_2_0
  ( fltk_input_set_value,
   "fltk-input::set-value",
   "Sets the current value.",
   widget, value,
   fltk_input, str )
{
  GFL_ASSERT_2 (fltk_input_set_value, widget, value);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  w->value(SCM_CHARS(value));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
  ( fltk_input_set_when,
   "fltk-input::set-when",
   "Sets when a callback is done.",
   widget, when,
   fltk_input, fltk_when )
{
  GFL_ASSERT_2 (fltk_input_set_when, widget, when);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  w->when(gh_scm2int(when));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_input_get_when,
   "fltk-input::get-when",
   "Gets when a callback is done.",
   widget, 
   fltk_input,
   when,
   fltk_when )
{
  GFL_ASSERT_1 (fltk_input_get_when, widget);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  return gh_int2scm((int)w->when());
}

GFL_PROCEDURE_2_0
  ( fltk_input_set_text_color,
   "fltk-input::set-text-color",
   "Sets the color of the text in the input field.",
   widget, color,
   fltk_input, fltk_color )
{
  GFL_ASSERT_2 (fltk_input_set_text_color, widget, color);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  w->textcolor(gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_input_get_text_color,
   "fltk-input::get-text-color",
   "Gets the color of the text in the input field.",
   widget,
   fltk_input, 
   color,
   fltk_color )
{
  GFL_ASSERT_1 (fltk_input_get_text_color, widget);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  return gh_int2scm((int)w->textcolor());
}

GFL_PROCEDURE_2_0
  ( fltk_input_set_text_font,
   "fltk-input::set-text-font",
   "Sets the font used for text in the input field.",
   widget, font,
   fltk_input, fltk_font )
{
  GFL_ASSERT_2 (fltk_input_set_text_font, widget, font);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  w->textfont(gh_scm2int(font));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_input_get_text_font,
   "fltk-input::get-text-font",
   "Gets the font used for text in the input field.",
   widget, 
   fltk_input,
   font,
   fltk_font )
{
  GFL_ASSERT_1 (fltk_input_get_text_font, widget);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  return gh_int2scm((int)w->textfont());
}

GFL_PROCEDURE_2_0
  ( fltk_input_set_text_size,
   "fltk-input::set-text-size",
   "Sets the size of the font used for text in the input field.",
   widget, size,
   fltk_input, int )
{
  GFL_ASSERT_2 (fltk_input_set_text_size, widget, size);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  w->textsize(gh_scm2int(size));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_input_get_text_size,
   "fltk-input::get-text-size",
   "Gets the size of the font used for text in the input field.",
   widget, 
   fltk_input, 
   size,
   int )
{
  GFL_ASSERT_1 (fltk_input_get_text_size, widget);
  Fl_Input *w = (Fl_Input *) gh_cdr (widget);
  return gh_int2scm((int)w->textsize());
}

/*
   4.  Initialization Hook:
*/

void
fltk_input_init ()
{
  GFL_NEW_TYPE (fltk_input);
  GFL_NEW_PROCEDURE (fltk_input_new);
  GFL_NEW_PROCEDURE (fltk_input_set_cursor_color);
  GFL_NEW_PROCEDURE (fltk_input_get_cursor_color);
  GFL_NEW_PROCEDURE (fltk_input_index);
  GFL_NEW_PROCEDURE (fltk_input_size);
  GFL_NEW_PROCEDURE (fltk_input_get_value);
  GFL_NEW_PROCEDURE (fltk_input_set_value);
  GFL_NEW_PROCEDURE (fltk_input_get_when);
  GFL_NEW_PROCEDURE (fltk_input_set_when);
  GFL_NEW_PROCEDURE (fltk_input_set_text_color);
  GFL_NEW_PROCEDURE (fltk_input_get_text_color);
  GFL_NEW_PROCEDURE (fltk_input_set_text_font);
  GFL_NEW_PROCEDURE (fltk_input_get_text_font);
  GFL_NEW_PROCEDURE (fltk_input_set_text_size);
  GFL_NEW_PROCEDURE (fltk_input_get_text_size);
}
