/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Input_.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Input_ *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Input_) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Input_ *widget = (Fl_Input_ *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Input_ ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-input-base", 
		   "Description of fltk-base..", 
		   fltk_input_base, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

/*
static SCM
fltk_input_base_wordboundary(SCM widget_smob, SCM a)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base::wordboundary");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  if (gh_int2scm((uchar)widget->wordboundary(aa)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_lineboundary(SCM widget_smob, SCM a)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base::lineboundary");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  if (gh_int2scm((uchar)widget->lineboundary(aa)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_drawtext(SCM widget_smob, SCM a, SCM b, SCM c, SCM d)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base::drawtext");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  int bb = gh_scm2int(b);
  int cc = gh_scm2int(c);
  int dd = gh_scm2int(d);

  if (gh_int2scm((uchar)widget->drawtext(aa, bb, cc, dd)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_handletext(SCM widget_smob, SCM a, SCM b, SCM c, SCM d, SCM e)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base::handletext");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  int bb = gh_scm2int(b);
  int cc = gh_scm2int(c);
  int dd = gh_scm2int(d);
  int ee = gh_scm2int(e);

  if (gh_int2scm((uchar)widget->handletext(aa, bb, cc, dd, ee)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_up_down_position(SCM widget_smob, SCM a, SCM b)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base::up-down-position");

  widget = (Fl_Input_ *) gh_cdr (widget_smob);

  int aa = gh_scm2int(a);
  int bb = gh_scm2int(b);

  if (gh_int2scm((uchar)widget->up_down_position(aa, bb)))
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

static SCM
fltk_input_base_maybe_do_callback(SCM widget_smob)
{
  Fl_Input_ *widget;

  SCM_ASSERT ((SCM_NIMP (widget_smob)
               && gfltk_class_check(widget_smob, fltk_input_base_tag),
              widget_smob, SCM_ARG1, "fltk-input-base::maybe-do-callback");

  //TODO: ASSERT for double argument
  widget = (Fl_Input_ *) gh_cdr (widget_smob);
  widget->maybe_do_callback();

  return SCM_UNSPECIFIED;
}
*/

GFL_PROCEDURE_1_1
  ( fltk_input_base_get_position,
   "fltk-input-base::get-position",
   "Gets the cursor position.",
   widget,
   fltk_input_base,
   position,
   int )
{
  GFL_ASSERT_1 (fltk_input_base_get_position, widget);
  Fl_Input_ *w = (Fl_Input_ *) gh_cdr (widget);
  return gh_int2scm(w->position());
}

GFL_PROCEDURE_3_0
  ( fltk_input_base_set_position,
    "fltk-input-base::set-position",
    "Sets the new cursor position and the new mark position. "
    "Returns true if new position is different from old one",
    widget, new_position, new_mark,
    fltk_input_base, int, int )
{
  GFL_ASSERT_3 (fltk_input_base_set_position, widget, new_position, new_mark);
  Fl_Widget *w = (Fl_Input_ *) gh_cdr (widget);
  w->position(gh_scm2int(new_position), gh_scm2int(new_mark));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_5_1
( fltk_input_base_replace,
  "fltk-input-base::replace",
  "Deletes the region between a and b, and then inserts the string \"insert\" "
  "at that point and leaves the mark and position after the insertion.  If "
  "\"length\" is non-zero, then \"length\" number of characters will be inserted.",
  widget, a, b, insert, length,
  fltk_input_base, int, int, str, int,
  changed,
  bool)
{
  GFL_ASSERT_5 (fltk_input_base_replace, widget, a, b, insert, length);
  Fl_Input_ *w = (Fl_Input_ *) gh_cdr (widget);
  if (w->replace(gh_scm2int(a), gh_scm2int(b), 
		 gh_scm2newstr(insert, NULL), gh_scm2int(length)))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
( fltk_input_base_copy,
  "fltk-input-base::copy",
  "Puts the current selection between mark and position into the clipboard.",
  widget,
  fltk_input_base,
  changed,
  bool)
{
  GFL_ASSERT_1 (fltk_input_base_copy, widget);
  Fl_Input_ *w = (Fl_Input_ *) gh_cdr (widget);
  if (w->copy())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
( fltk_input_base_undo,
  "fltk-input-base::undo",
  "Does undo of several previous calls to replace.  Returns true if change was made.",
  widget,
  fltk_input_base,
  changed,
  bool)
{
  GFL_ASSERT_1 (fltk_input_base_undo, widget);
  Fl_Input_ *w = (Fl_Input_ *) gh_cdr (widget);
  if (w->undo())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
( fltk_input_base_copy_cuts,
  "fltk-input-base::copy-cuts",
  "Copys all the previous contiguous cuts from the undo information to the clipboard",
  widget,
  fltk_input_base,
  changed,
  bool )
{
  GFL_ASSERT_1 (fltk_input_base_copy_cuts, widget);
  Fl_Input_ *w = (Fl_Input_ *) gh_cdr (widget);
  if (w->copy_cuts())
    return SCM_BOOL_T;
  return SCM_BOOL_F;    
}


void
fltk_input_base_init ()
{
  GFL_NEW_TYPE (fltk_input_base);
  // add new methods
  //gh_new_procedure1_0 ("fltk-input-base::wordboundary", 
  //		       fltk_input_base_wordboundary);
  //  gh_new_procedure1_0 ("fltk-input-base::lineboundary", 
  //		       fltk_input_base_lineboundary);
  //  gh_new_procedure5_0 ("fltk-input-base::drawtext", 
  //		       fltk_input_base_drawtext);
  //  gh_new_procedure6_0 ("fltk-input-base::handletext", 
  //		       fltk_input_base_handletext);
  //  gh_new_procedure3_0 ("fltk-input-base::up-down-position", 
  //		       fltk_input_base_up_down_position);
  //  gh_new_procedure1_0 ("fltk-input-base::maybe-do-callback", 
  //		       fltk_input_base_maybe_do_callback);
  GFL_NEW_PROCEDURE (fltk_input_base_get_position);
  GFL_NEW_PROCEDURE (fltk_input_base_set_position);
  GFL_NEW_PROCEDURE (fltk_input_base_replace);
  GFL_NEW_PROCEDURE (fltk_input_base_copy);
  GFL_NEW_PROCEDURE (fltk_input_base_undo);
  GFL_NEW_PROCEDURE (fltk_input_base_copy_cuts);
}

/*
void
fltk_input_base_subclass (char *s, long tag)
{
  gfltk_class_register(tag, fltk_input_base_tag);
  fltk_widget_subclass(s, tag);

      sprintf (t, "%s-wordboundary", s);
      gh_define (t, scm_symbol_value0("fltk-input-base::wordboundary"));
      sprintf (t, "%s-lineboundary", s);
      gh_define (t, scm_symbol_value0("fltk-input-base::lineboundary"));
      sprintf (t, "%s-drawtext", s);
      gh_define (t, scm_symbol_value0("fltk-input-base::drawtext"));
      sprintf (t, "%s-handletext", s);
      gh_define (t, scm_symbol_value0("fltk-input-base::handletext"));
      sprintf (t, "%s-up-down-position", s);
      gh_define (t, scm_symbol_value0("fltk-input-base::up-down-position"));
      sprintf (t, "%s-maybe-do-callback", s);
      gh_define (t, scm_symbol_value0("fltk-input-base::maybe-do-callback"));
  char t[120];
  sprintf (t, "%s-get-position", s);
  gh_define (t, scm_symbol_value0("fltk-input-base::get-position"));
  sprintf (t, "%s-set-position", s);
  gh_define (t, scm_symbol_value0("fltk-input-base::set-position"));
  sprintf (t, "%s-replace", s);
  gh_define (t, scm_symbol_value0("fltk-input-base::replace"));
  sprintf (t, "%s-copy", s);
  gh_define (t, scm_symbol_value0("fltk-input-base::copy"));
  sprintf (t, "%s-undo", s);
  gh_define (t, scm_symbol_value0("fltk-input-base::undo"));
  sprintf (t, "%s-copy-cuts", s);
  gh_define (t, scm_symbol_value0("fltk-input-base::copy-cuts"));
}
*/
