/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Menu_Button.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Menu_Button *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (GFL_Widget) + sizeof (Fl_Menu_Button));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Menu_Button *widget = (Fl_Menu_Button *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Menu_Button ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-menu-button",
		   "fltk-menu-button is a button that when pushed, pops up a menu.",
		   fltk_menu_button, fltk_menu,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_menu_button_new,
   "fltk-menu-button::new",
   "Creates a new fltk-menu-button with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   menu_button,
   fltk_menu_button )
{
  GFL_ASSERT_5 (fltk_menu_button_new, x, y, w, h, label);
  Fl_Menu_Button * widget = new Fl_Menu_Button(gh_scm2int(x), gh_scm2int(y), 
			       gh_scm2int(w), gh_scm2int(h),
			       gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_menu_button, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1 
  ( fltk_menu_button_popup,
   "fltk-menu-button::popup",
   "Causes the menu to be popped up, and returns the selected menu item, or "
   "null if none was selected.",
   widget,
   fltk_menu_button,
   menu_item,
   fltk_menu_item )
{
  GFL_ASSERT_1 (fltk_menu_button_popup, widget);
  Fl_Menu_Button *w = (Fl_Menu_Button *) gh_cdr (widget);
  // TODO: FIXME - NEED AN Fl_Menu_Item class
  return gh_long2scm((int)w->popup());
}

GFL_PROCEDURE_2_0
  ( fltk_menu_button_set_type,
   "fltk-menu-button::set-type",
   "Sets the type of fltk-menu-button.  Valid types include: POPUP1, POPUP2, POPUP3, POPUP12, POPUP13, POPUP23, and POPUP123.",
   widget, type,
   fltk_menu_button, fltk_menu_buttontype )
{
  GFL_ASSERT_2 (fltk_menu_button_set_type, widget, type);
  Fl_Menu_Button *w = (Fl_Menu_Button *) gh_cdr (widget);
  w->type(gh_scm2int(type));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_menu_button_init ()
{
  GFL_NEW_TYPE (fltk_menu_button);
  GFL_NEW_PROCEDURE (fltk_menu_button_new);
  GFL_NEW_PROCEDURE (fltk_menu_button_popup);
  GFL_NEW_PROCEDURE (fltk_menu_button_set_type);
  // gh_define("POPUP0", gh_long2scm(POPUP0));
  enum {POPUP1 = 1, POPUP2, POPUP12, POPUP3, POPUP13, POPUP23, POPUP123};
  gh_define("POPUP1", gh_long2scm(POPUP1));
  gh_define("POPUP2", gh_long2scm(POPUP2));
  gh_define("POPUP3", gh_long2scm(POPUP3));

}
