/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Output.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Output *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Output) + sizeof (Fl_Output));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Output *widget = (Fl_Output *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Output ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-output", 
		   "fltk-output displays a peice of text.",
		   fltk_output, fltk_input_base,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_output_new,
   "fltk-output::new",
   "Creates a new fltk-output with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   output,
   fltk_output )
{
  GFL_ASSERT_5 (fltk_output_new, x, y, w, h, label);

  Fl_Output * widget = new Fl_Output(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_output, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_2_0
  ( fltk_output_set_cursor_color,
   "fltk-output::set-cursor-color",
   "Sets the color of the cursor.",
   widget, color,
   fltk_output, fltk_color )
{
  GFL_ASSERT_2 (fltk_output_set_cursor_color, widget, color);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  w->cursor_color((Fl_Boxtype)gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_output_get_cursor_color,
   "fltk-output::get-cursor-color",
   "Gets the color of the cursor.",
   widget,
   fltk_output, 
   color,
   fltk_color )
{
  GFL_ASSERT_1 (fltk_output_get_cursor_color, widget);
  Fl_Output * w = (Fl_Output *) gh_cdr (widget);
  return gh_int2scm((int)w->cursor_color());
}

GFL_PROCEDURE_2_1
  ( fltk_output_index,
   "fltk-output::index",
   "Gets the Nth character in the value string.",
   widget, N,
   fltk_output, int,
   color,
   fltk_color )
{
  GFL_ASSERT_2 (fltk_output_index, widget, N);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  if (N < w->size())
    return gh_char2scm(w->index(gh_scm2int(N)));
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
  ( fltk_output_size,
   "fltk-output::size",
   "Returns the number of characters in the value string.",
   widget, 
   fltk_output,
   N,
   int )
{
  GFL_ASSERT_1 (fltk_output_size, widget);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  return gh_int2scm(w->size());
}

GFL_PROCEDURE_1_1
  ( fltk_output_get_value,
   "fltk-output::get-value",
   "Gets the current value.",
   widget, 
   fltk_output,
   value,
   str )
{
  GFL_ASSERT_1 (fltk_output_get_value, widget);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  return gh_str02scm((char *)w->value());
}

GFL_PROCEDURE_2_0
  ( fltk_output_set_value,
   "fltk-output::set-value",
   "Sets the current value.",
   widget, value,
   fltk_output, str )
{
  GFL_ASSERT_2 (fltk_output_set_value, widget, value);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  w->value(SCM_CHARS(value));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
  ( fltk_output_set_text_color,
   "fltk-output::set-text-color",
   "Sets the color of the text in the output field.",
   widget, color,
   fltk_output, fltk_color )
{
  GFL_ASSERT_2 (fltk_output_set_text_color, widget, color);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  w->textcolor(gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_output_get_text_color,
   "fltk-output::get-text-color",
   "Gets the color of the text in the output field.",
   widget,
   fltk_output, 
   color,
   fltk_color )
{
  GFL_ASSERT_1 (fltk_output_get_text_color, widget);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  return gh_int2scm((int)w->textcolor());
}

GFL_PROCEDURE_2_0
  ( fltk_output_set_text_font,
   "fltk-output::set-text-font",
   "Sets the font used for text in the output field.",
   widget, font,
   fltk_output, fltk_font )
{
  GFL_ASSERT_2 (fltk_output_set_text_font, widget, font);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  w->textfont(gh_scm2int(font));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_output_get_text_font,
   "fltk-output::get-text-font",
   "Gets the font used for text in the output field.",
   widget, 
   fltk_output,
   font,
   fltk_font )
{
  GFL_ASSERT_1 (fltk_output_get_text_font, widget);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  return gh_int2scm((int)w->textfont());
}

GFL_PROCEDURE_2_0
  ( fltk_output_set_text_size,
   "fltk-output::set-text-size",
   "Sets the size of the font used for text in the output field.",
   widget, size,
   fltk_output, int )
{
  GFL_ASSERT_2 (fltk_output_set_text_size, widget, size);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  w->textsize(gh_scm2int(size));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_output_get_text_size,
   "fltk-output::get-text-size",
   "Gets the size of the font used for text in the output field.",
   widget, 
   fltk_output, 
   size,
   int )
{
  GFL_ASSERT_1 (fltk_output_get_text_size, widget);
  Fl_Output *w = (Fl_Output *) gh_cdr (widget);
  return gh_int2scm((int)w->textsize());
}

/*
   4.  Initialization Hook:
*/

void
fltk_output_init ()
{
  GFL_NEW_TYPE (fltk_output);
  GFL_NEW_PROCEDURE (fltk_output_new);
  GFL_NEW_PROCEDURE (fltk_output_set_cursor_color);
  GFL_NEW_PROCEDURE (fltk_output_get_cursor_color);
  GFL_NEW_PROCEDURE (fltk_output_index);
  GFL_NEW_PROCEDURE (fltk_output_size);
  GFL_NEW_PROCEDURE (fltk_output_get_value);
  GFL_NEW_PROCEDURE (fltk_output_set_value);
  GFL_NEW_PROCEDURE (fltk_output_set_text_color);
  GFL_NEW_PROCEDURE (fltk_output_get_text_color);
  GFL_NEW_PROCEDURE (fltk_output_set_text_font);
  GFL_NEW_PROCEDURE (fltk_output_get_text_font);
  GFL_NEW_PROCEDURE (fltk_output_set_text_size);
  GFL_NEW_PROCEDURE (fltk_output_get_text_size);
}
