/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Scrollbar.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Scrollbar *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Scrollbar) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Scrollbar *widget = (Fl_Scrollbar *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Scrollbar ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-scrollbar",
		   "fltk-scrollbar displays a slider with arrow buttons at "
		   "the ends of the scrollbar",
		   fltk_scrollbar, fltk_slider,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1
( fltk_scrollbar_new,
  "fltk-scrollbar::new ",
  "Creates a new fltk-scrollbar.",
  x, y, w, h, label,
  int, int, int, int, str,
  widget,
  fltk_scrollbar)
{
  GFL_ASSERT_5 (fltk_scrollbar_new, x, y, w, h, label);
  Fl_Scrollbar *widget = new Fl_Scrollbar(gh_scm2int(x), gh_scm2int(y), 
				    gh_scm2int(w), gh_scm2int(h),
				    gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_scrollbar, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
( fltk_scrollbar_get_linesize, 
  "fltk-scrollbar::get-linesize", 
  "Gets the linesize increment which is used to determine how many "
  "pixels the arrow keys will scroll the display.", 
  widget, 
  fltk_scrollbar, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_scrollbar_get_linesize, widget);  
  Fl_Scrollbar *wi = (Fl_Scrollbar *) gh_cdr (widget);
  return gh_int2scm(wi->linesize());
}

GFL_PROCEDURE_2_0
( fltk_scrollbar_set_linesize, 
  "fltk-scrollbar::set-linesize", 
  "Sets the linesize increment which is used to determine how many "
  "pixels the arrow keys will scroll the display.", 
  widget, a, 
  fltk_scrollbar, int)
{
  GFL_ASSERT_2(fltk_scrollbar_set_linesize, widget, a);
  Fl_Scrollbar *wi = (Fl_Scrollbar *) gh_cdr (widget);
  wi->linesize(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_scrollbar_get_value, 
  "fltk-scrollbar::get-value", 
  "Gets the integer value of the scrollbar.  Use fltk-slider-get-value "
  "to return the float value of the scrollbar.", 
  widget, 
  fltk_scrollbar, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_scrollbar_get_value, widget);
  Fl_Scrollbar *wi = (Fl_Scrollbar *) gh_cdr (widget);
  return gh_int2scm(wi->value());
}

GFL_PROCEDURE_5_0
( fltk_scrollbar_set_value, 
  "fltk-scrollbar::set-value", 
  "Sets the value, range, and slider_size to make a variable-sized scrollbar.", 
  widget, position, size, top, total, 
  fltk_scrollbar, int, int, int, int)
{
  GFL_ASSERT_5(fltk_scrollbar_set_value, widget, position, size, top, total);
  Fl_Scrollbar *wi = (Fl_Scrollbar *) gh_cdr (widget);
  gh_int2scm(wi->value(gh_scm2int(position), gh_scm2int(size), 
		       gh_scm2int(top), gh_scm2int(total)));
  return SCM_UNSPECIFIED;
}

void
fltk_scrollbar_init ()
{
  GFL_NEW_TYPE (fltk_scrollbar);
  GFL_NEW_PROCEDURE (fltk_scrollbar_new);
  GFL_NEW_PROCEDURE (fltk_scrollbar_get_linesize);
  GFL_NEW_PROCEDURE (fltk_scrollbar_set_linesize);
  GFL_NEW_PROCEDURE (fltk_scrollbar_get_value);
  GFL_NEW_PROCEDURE (fltk_scrollbar_set_value);
}
