/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Secret_Input.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Secret_Input *) SCM_CDR (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Secret_Input) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Secret_Input *widget = (Fl_Secret_Input *) SCM_CDR (widget_smob);
  scm_puts ("#<widget Fl_Secret_Input ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-secret-input", 
		   "fltk-secret-input is a subclass of fltk-input that hides the "
		   "the input text.",
		   fltk_secret_input, fltk_input,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_secret_input_new,
   "fltk-secret-input::new",
   "Creates a new fltk-secret-input with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   input,
   fltk_secret_input )
{
  GFL_ASSERT_5 (fltk_secret_input_new, x, y, w, h, label);

  Fl_Secret_Input * widget = new Fl_Secret_Input(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_secret_input, widget_smob, widget);
  return widget_smob;
}

/*
   4.  Initialization Hook:
*/

void
fltk_secret_input_init ()
{
  GFL_NEW_TYPE (fltk_secret_input);
  GFL_NEW_PROCEDURE (fltk_secret_input_new);
}

