/*
  Copyright 1999 by James Dean Palmer.
  
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
  
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.
  
  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Select_Browser.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Select_Browser *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Select_Browser) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Select_Browser *widget = (Fl_Select_Browser *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Select_Browser ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}
/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-select-browser", 
		   "fltk-select-browser is a subclass of fltk-browser which "
		   "lets the user select a single item, or no items",
		   fltk_select_browser, fltk_browser, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_select_browser_new,
  "fltk-select-browser::new",
  "Creates a new fltk-select-browser with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  select_browser,
  fltk_select_browser )
{
  GFL_ASSERT_5 (fltk_select_browser_new, x, y, w, h, label);
  Fl_Select_Browser * widget = new Fl_Select_Browser(gh_scm2int(x), gh_scm2int(y), 
						     gh_scm2int(w), gh_scm2int(h),
						     gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_select_browser, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
( fltk_select_browser_deselect, 
  "fltk-select-browser::deselect", 
  "Deselects all text.", 
  widget, 
  fltk_select_browser, 
  rval, 
  bool)
{
  GFL_ASSERT_1(fltk_select_browser_deselect, widget);
  Fl_Select_Browser *wi = (Fl_Select_Browser *) gh_cdr (widget);
  if (wi->deselect())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}


GFL_PROCEDURE_2_0
( fltk_select_browser_select, 
  "fltk-select-browser::select", 
  "Selects line N.  Multiple selections are undefined.", 
  widget, N, 
  fltk_select_browser, int)
{
  GFL_ASSERT_2 (fltk_select_browser_select, widget, N);
  Fl_Select_Browser *wi = (Fl_Select_Browser *) gh_cdr (widget);
  wi->select(gh_scm2int(N), 1);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_1
( fltk_select_browser_selected, 
  "fltk-select-browser::selected", 
  "Returns true if line N is selected.", 
  widget, N, 
  fltk_select_browser, int, 
  rval, 
  bool)
{
  GFL_ASSERT_2(fltk_select_browser_selected, widget, N);
  Fl_Select_Browser *wi = (Fl_Select_Browser *) gh_cdr (widget);
  if (wi->selected(gh_scm2int(N)))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
( fltk_select_browser_set_value, 
  "fltk-select-browser::set-value", 
  "Sets the selected line to N.", 
  widget, N, 
  fltk_select_browser, int)
  
{
  GFL_ASSERT_2(fltk_select_browser_set_value, widget, N);
  Fl_Select_Browser *wi = (Fl_Select_Browser *) gh_cdr (widget);
  wi->value(gh_scm2int(N));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_select_browser_get_value, 
  "fltk-select-browser::get-value", 
  "Gets the selected line.", 
  widget, 
  fltk_select_browser, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_select_browser_get_value, widget);
  Fl_Select_Browser *wi = (Fl_Select_Browser *) gh_cdr (widget);
  return gh_int2scm(wi->value());
}

/*
   4.  Initialization Hook:
*/

void
fltk_select_browser_init ()
{
  GFL_NEW_TYPE (fltk_select_browser);
  GFL_NEW_PROCEDURE (fltk_select_browser_new);
  GFL_NEW_PROCEDURE (fltk_select_browser_deselect);
  GFL_NEW_PROCEDURE (fltk_select_browser_select);
  GFL_NEW_PROCEDURE (fltk_select_browser_selected);
  GFL_NEW_PROCEDURE (fltk_select_browser_set_value);
  GFL_NEW_PROCEDURE (fltk_select_browser_get_value);
}
