/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Slider.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Slider *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Slider) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Slider *widget = (Fl_Slider *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Slider ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-slider", 
		   "fltk-slider provides a sliding bar inside a box which is "
		   "also known as a scroll bar.",
		   fltk_slider, fltk_valuator, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1
( fltk_slider_new,
  "fltk-slider::new ",
  "Creates a new fltk-slider with the given position, size and lavel.",
  x, y, w, h, label,
  int, int, int, int, str,
  widget,
  fltk_slider)
{
  GFL_ASSERT_5 (fltk_slider_new, x, y, w, h, label);
  Fl_Slider *widget = new Fl_Slider(gh_scm2int(x), gh_scm2int(y), 
				    gh_scm2int(w), gh_scm2int(h),
				    gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_slider, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_2_0
( fltk_slider_set_type, 
  "fltk-slider::set-type", 
  "Sets the slider type.  Valid types include: FL_VERTICAL, "
  "FL_HORIZONTAL, FL_VERT_FILL_SLIDER, FL_HOR_FILL_SLIDER, "
  "FL_VERT_NICE_SLIDER and FL_HOR_NICE_SLIDER.", 
  widget, a, 
  fltk_slider, int)
{
  GFL_ASSERT_2(fltk_slider_set_type, widget, a);
  Fl_Slider *wi = (Fl_Slider *) gh_cdr (widget);
  wi->type((uchar)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_slider_get_type, 
  "fltk-slider::get-type", 
  "Gets the slider type.", 
  widget, 
  fltk_slider, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_slider_get_type, widget);  
  Fl_Slider *wi = (Fl_Slider *) gh_cdr (widget);
  return gh_int2scm(wi->type());
}

GFL_PROCEDURE_2_0
  ( fltk_slider_set_slider, 
   "fltk-slider::set-slider", 
   "Sets the type of box to draw for the moving part of the slider.", 
   widget, a, 
   fltk_slider, int)
{
  GFL_ASSERT_2(fltk_slider_set_slider, widget, a);
  Fl_Slider *wi = (Fl_Slider *) gh_cdr (widget);
  wi->slider((Fl_Boxtype)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_slider_get_slider, 
   "fltk-slider::get-slider", 
   "Gets the type of box to draw for the moving part of the slider.", 
   widget, 
   fltk_slider, 
   rval, 
   int)
{
  GFL_ASSERT_1(fltk_slider_get_slider, widget);
  Fl_Slider *wi = (Fl_Slider *) gh_cdr (widget);
  return gh_int2scm(wi->slider());
}


GFL_PROCEDURE_2_0
  ( fltk_slider_set_slider_size, 
   "fltk-slider::set-slider-size", 
   "Sets the size of the moving part of the slider as a fraction of the entire box.", 
   widget, a, 
   fltk_slider, double)
{
  GFL_ASSERT_2(fltk_slider_set_slider_size, widget, a);
  Fl_Slider *wi = (Fl_Slider *) gh_cdr (widget);
  wi->slider_size(gh_scm2double(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_slider_get_slider_size, 
   "fltk-slider::get-slider-size", 
   "Gets teh size of the moving part of the slider as a fraction of the entire box.", 
   widget, 
   fltk_slider, 
   rval, 
   double)
{
  GFL_ASSERT_1(fltk_slider_get_slider_size, widget);
  Fl_Slider *wi = (Fl_Slider *) gh_cdr (widget);
  return gh_double2scm(wi->slider_size());
}

GFL_PROCEDURE_5_1
  ( fltk_slider_scrollvalue, 
   "fltk-slider::scrollvalue", 
  "Sets the value, range, and slider_size to make a variable-sized scrollbar.", 
  widget, position, size, top, total, 
   fltk_slider, int, int, int, int, 
   rval, 
   int)
{
  GFL_ASSERT_5(fltk_slider_scrollvalue, widget, position, size, top, total);
  Fl_Slider *wi = (Fl_Slider *) gh_cdr (widget);
  return gh_int2scm(wi->scrollvalue(gh_scm2int(position), 
				    gh_scm2int(size), 
				    gh_scm2int(top), 
				    gh_scm2int(total)));
}

/*
   4.  Initialization Hook:
*/

void
fltk_slider_init ()
{
  GFL_NEW_TYPE (fltk_slider);
  GFL_NEW_PROCEDURE (fltk_slider_new);
  GFL_NEW_PROCEDURE (fltk_slider_get_type);
  GFL_NEW_PROCEDURE (fltk_slider_set_type);
  GFL_NEW_PROCEDURE (fltk_slider_set_slider);
  GFL_NEW_PROCEDURE (fltk_slider_get_slider);
  GFL_NEW_PROCEDURE (fltk_slider_set_slider_size);
  GFL_NEW_PROCEDURE (fltk_slider_get_slider_size);
  GFL_NEW_PROCEDURE (fltk_slider_scrollvalue);
  GFL_NEW_PROCEDURE (fltk_slider_set_type);
  GFL_NEW_PROCEDURE (fltk_slider_get_type);
}
