/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Tabs.H>
#include <gFL/guile_fltk.h>
GFL_EXTERN_TYPE (fltk_widget);

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Tabs *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Tabs) + sizeof (Fl_Tabs));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Tabs *widget = (Fl_Tabs *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Tabs ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-tabs", 
		   "fltk-tabs is a fltk-group subclass that lets you resize "
		   "the children by dragging the border between them.",
		   fltk_tabs, fltk_group, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_tabs_new,
  "fltk-tabs::new",
  "Creates a new fltk-tabs with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  tabs,
  fltk_tabs )
{
  GFL_ASSERT_5 (fltk_tabs_new, x, y, w, h, label);
  Fl_Tabs * widget = new Fl_Tabs(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_tabs, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_2_0
(fltk_tabs_set_value,
 "fltk-tabs::set-value",
 "Sets the currently visible widget / tab.",
 tabs, widget,
 fltk_tabs, fltk_widget )
{
  GFL_ASSERT_2 (fltk_tabs_set_value, tabs, widget);
  Fl_Tabs *r = (Fl_Tabs *) gh_cdr (tabs);
  r->value ((Fl_Widget *) gh_cdr (widget));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
(fltk_tabs_get_value,
 "fltk-tabs::get-value",
 "Sets the currently visible widget / tab.",
 tabs,
 fltk_tabs, 
 widget,
 fltk_widget )
{
  GFL_ASSERT_1 (fltk_tabs_get_value, tabs);
  Fl_Tabs *r = (Fl_Tabs *) gh_cdr (tabs);
  return gfl_lookup (r->value());
}

/*
   4.  Initialization Hook:
*/

void
fltk_tabs_init ()
{
  GFL_NEW_TYPE (fltk_tabs);
  GFL_NEW_PROCEDURE (fltk_tabs_new);
  GFL_NEW_PROCEDURE (fltk_tabs_get_value);
  GFL_NEW_PROCEDURE (fltk_tabs_set_value);
}
