/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Timer.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Timer *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Timer) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Timer *widget = (Fl_Timer *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Timer ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-timer",
		   "fltk-timer makes a time out call every 1/5 second.",
		   fltk_timer, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_6_1 
  ( fltk_timer_new,
   "fltk-timer::new",
   "Creates a new fltk-timer with the given type, position, size, and "
   "label string.",
   type, x, y, w, h, label,
   fltk_boxtype, int, int, int, int, str,
   box,
   fltk_timer )
{
  GFL_ASSERT_6 (fltk_timer_new, type, x, y, w, h, label);
  Fl_Timer * widget = new Fl_Timer(gh_scm2int(type), gh_scm2int(x), gh_scm2int(y), 
				   gh_scm2int(w), gh_scm2int(h),
				   gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_timer, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_2_0
  ( fltk_timer_set_suspended,
   "fltk-timer::set-suspended",
   "Sets whether the timer is suspended.",
   widget, suspended,
   fltk_timer, int )
{
  GFL_ASSERT_2 (fltk_timer_set_suspended, widget, suspended);
  Fl_Timer *w = (Fl_Timer *) gh_cdr (widget);
  if (suspended == SCM_BOOL_F)
    w->suspended(0);
  else
    w->suspended(1);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_timer_get_suspended,
   "fltk-timer::get-suspended",
   "Gets whether the timer is suspended.",
   widget,
   fltk_timer,
   suspended,
   int )
{
  GFL_ASSERT_1 (fltk_timer_get_suspended, widget);
  Fl_Timer *w = (Fl_Timer *) gh_cdr (widget);
  if (w->suspended())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
  ( fltk_timer_set_direction,
   "fltk-timer::set-direction",
   "Sets the direction of the timer.  If the direction is zero "
   "then the timer will count up, otherwise it will count down.",
   widget, direction,
   fltk_timer, int )
{
  GFL_ASSERT_2 (fltk_timer_set_direction, widget, direction);
  Fl_Timer *w = (Fl_Timer *) gh_cdr (widget);
  if (direction == SCM_BOOL_F)
    w->direction(0);
  else
    w->direction(1);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_timer_get_direction,
   "fltk-timer::get-direction",
   "Gets the direction of the timer.  If the direction is zero "
   "then the timer will count up, otherwise it will count down.",
   widget,
   fltk_timer,
   direction,
   int )
{
  GFL_ASSERT_1 (fltk_timer_get_direction, widget);
  Fl_Timer *w = (Fl_Timer *) gh_cdr (widget);
  if (w->direction())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
  ( fltk_timer_set_value,
   "fltk-timer::set-value",
   "Sets the current timer value.",
   widget, value,
   fltk_timer, float )
{
  GFL_ASSERT_2 (fltk_timer_set_value, widget, value);
  Fl_Timer *w = (Fl_Timer *) gh_cdr (widget);
  w->value (gh_scm2double(value));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_timer_get_value,
   "fltk-timer::get-value",
   "Gets the current timer value.",
   widget,
   fltk_timer, 
   value,
   float )
{
  GFL_ASSERT_1 (fltk_timer_get_value, widget);
  Fl_Timer *w = (Fl_Timer *) gh_cdr (widget);
  return gh_double2scm(w->value());
}

/*
   4.  Initialization Hook:
*/

void
fltk_timer_init ()
{
  GFL_NEW_TYPE (fltk_timer);
  GFL_NEW_PROCEDURE (fltk_timer_new);
  GFL_NEW_PROCEDURE (fltk_timer_set_value);
  GFL_NEW_PROCEDURE (fltk_timer_get_value);
  GFL_NEW_PROCEDURE (fltk_timer_set_direction);
  GFL_NEW_PROCEDURE (fltk_timer_get_direction);
  GFL_NEW_PROCEDURE (fltk_timer_set_suspended);
  GFL_NEW_PROCEDURE (fltk_timer_get_suspended);
}
