/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Valuator.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Valuator *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Valuator) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Valuator *widget = (Fl_Valuator *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Valuator ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-valuator", 
		   "fltk-valuator controls a single floating-point value.",
		   fltk_valuator, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_1_1
( fltk_valuator_get_value,
  "fltk-valuator::get-value",
  "Gets the current value",
  widget,
  fltk_valuator,
  value,
  double)
{
  GFL_ASSERT_1 (fltk_valuator_get_value, widget);
  Fl_Valuator * w = (Fl_Valuator *) gh_cdr (widget);
  return gh_double2scm(w->value());
}

GFL_PROCEDURE_2_0
( fltk_valuator_set_value,
  "fltk-valuator::set-value",
  "Sets the current value.",
  widget, value,
  fltk_valuator, double )
{
  GFL_ASSERT_2 (fltk_valuator_set_value, widget, value);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  w->value(gh_scm2double(value));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_valuator_get_minimum,
  "fltk-valuator::get-minimum",
  "Gets the minimum value.",
  widget,
  fltk_valuator, 
  value,
  double)
{
  GFL_ASSERT_1 (fltk_valuator_get_minimum, widget);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  return gh_double2scm(w->minimum());
}

GFL_PROCEDURE_2_0
( fltk_valuator_set_minimum,
  "fltk-valuator::set-minimum",
  "Sets the minimum value.",
  widget, value,
  fltk_valuator, double)
{
  GFL_ASSERT_2 (fltk_valuator_set_minimum, widget, value);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  w->minimum (gh_scm2double(value));
  return SCM_UNSPECIFIED;
}


GFL_PROCEDURE_1_1
( fltk_valuator_get_maximum,
  "fltk-valuator::get-maximum",
  "Gets the maximum value.",
  widget, 
  fltk_valuator,
  value,
  double)
{
  GFL_ASSERT_1 (fltk_valuator_get_maximum, widget);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  return gh_double2scm(w->maximum());
}

GFL_PROCEDURE_2_0
( fltk_valuator_set_maximum,
  "fltk-valuator::set-maximum",
  "Sets the maximum value.",
  widget, maximum,
  fltk_valuator, double)
{
  GFL_ASSERT_2 (fltk_valuator_set_maximum, widget, maximum);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  w->maximum(gh_scm2double(maximum));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_valuator_range,
  "fltk-valuator::range",
  "Sets the value range.",
  widget, maximum, minimum,
  fltk_valuator, double, double)
{
  GFL_ASSERT_3 (fltk_valuator_range, widget, maximum, minimum);
  Fl_Valuator * w = (Fl_Valuator *) gh_cdr (widget);
  w->range(gh_scm2double(minimum), gh_scm2double(maximum));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_valuator_get_step,
  "fltk-valuator::get-step",
  "Gets the step increment.",
  widget,
  fltk_valuator, 
  step,
  double)
{
  GFL_ASSERT_1 (fltk_valuator_get_step, widget);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  return gh_double2scm(w->step());
}

GFL_PROCEDURE_2_0
( fltk_valuator_set_step,
  "fltk-valuator::set-step",
  "Sets the step increment.",
  widget, step,
  fltk_valuator, double)
{
  GFL_ASSERT_2 (fltk_valuator_set_step, widget, step);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  w->step(gh_scm2double(step));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_valuator_format,
  "fltk-valuator::format",
  "Returns a string formatted to show enough digits of the current value "
  "with the current step value.",
  widget,
  fltk_valuator,
  value,
  str)
{
  GFL_ASSERT_1 (fltk_valuator_format, widget);
  char buffer[40];
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  w->format(buffer);
  return scm_makfrom0str(buffer);
}

GFL_PROCEDURE_2_1
( fltk_valuator_round,
  "fltk-valuator::round",
  "Round the passed value to the nearest step increment. Does nothing "
  "if step is zero.",
  widget, value,
  fltk_valuator, double,
  rounded_value,
  double)
{
  GFL_ASSERT_2 (fltk_valuator_round, widget, value);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  return gh_double2scm(w->round(gh_scm2double(value)));
}

GFL_PROCEDURE_2_1
( fltk_valuator_clamp,
  "fltk-valuator::clamp",
  "Clamp the passed value to the valuator range.",
  widget, value,
  fltk_valuator, double,
  clamped_value,
  double)
{
  GFL_ASSERT_2 (fltk_valuator_clamp, widget, value);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  return gh_double2scm(w->clamp(gh_scm2double(value)));
}

GFL_PROCEDURE_3_1
( fltk_valuator_increment,
  "fltk-valuator::increment",
  "Adds n times the step value to the passed value. If step was set to zero "
  "it uses fabs(maximum - minimum) / 100.",
  widget, value, N,
  fltk_valuator, double, int,
  incremented_value,
  double)
{
  GFL_ASSERT_3 (fltk_valuator_increment, widget, value, N);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  return gh_double2scm(w->increment(gh_scm2double(value), gh_scm2int(N)));
}

GFL_PROCEDURE_1_1
( fltk_valuator_get_changed,
  "fltk-valuator::get-changed",
  "Gets wether the value has been changed by the user moving the slider.",
  widget,
  fltk_valuator,
  value,
  bool)
{
  GFL_ASSERT_1 (fltk_valuator_get_changed, widget);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  if (w->changed())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
( fltk_valuator_set_changed,
  "fltk-valuator::set-changed",
  "Sets whether the widget has been changed.",
  widget, changed,
  fltk_valuator, bool)
{
  GFL_ASSERT_2 (fltk_valuator_set_changed, widget, changed);
  Fl_Valuator *w = (Fl_Valuator *) gh_cdr (widget);
  if (gh_scm2int(changed))
    w->set_changed();
  else
    w->clear_changed();
  return SCM_UNSPECIFIED;
}

void
fltk_valuator_init ()
{
  GFL_NEW_TYPE (fltk_valuator);
  GFL_NEW_PROCEDURE (fltk_valuator_get_value);
  GFL_NEW_PROCEDURE (fltk_valuator_set_value);
  GFL_NEW_PROCEDURE (fltk_valuator_get_minimum);
  GFL_NEW_PROCEDURE (fltk_valuator_set_minimum);
  GFL_NEW_PROCEDURE (fltk_valuator_get_maximum);
  GFL_NEW_PROCEDURE (fltk_valuator_set_maximum);
  GFL_NEW_PROCEDURE (fltk_valuator_range);
  GFL_NEW_PROCEDURE (fltk_valuator_get_step);
  GFL_NEW_PROCEDURE (fltk_valuator_set_step);
  GFL_NEW_PROCEDURE (fltk_valuator_format);
  GFL_NEW_PROCEDURE (fltk_valuator_round);
  GFL_NEW_PROCEDURE (fltk_valuator_clamp);
  GFL_NEW_PROCEDURE (fltk_valuator_increment);
  GFL_NEW_PROCEDURE (fltk_valuator_get_changed);
  GFL_NEW_PROCEDURE (fltk_valuator_set_changed);
}
