/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Value_Output.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Value_Output *) SCM_CDR (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Value_Output) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Value_Output *widget = (Fl_Value_Output *) SCM_CDR (widget_smob);
  scm_puts ("#<widget Fl_Value_Output ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-value-output", 
		   "fltk-value-output displays a floating point value.",
		   fltk_value_output, fltk_output,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1
( fltk_value_output_new,
  "fltk-value-output::new ",
  "Creates a new fltk-value-output.",
  x, y, w, h, label,
  int, int, int, int, str,
  widget,
  fltk_output)
{
  GFL_ASSERT_5 (fltk_value_output_new, x, y, w, h, label);
  Fl_Value_Output *widget = new Fl_Value_Output(gh_scm2int(x), gh_scm2int(y), 
						gh_scm2int(w), gh_scm2int(h),
						gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_output, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
  ( fltk_value_output_get_soft, 
   "fltk-value-output::get-soft", 
   "Gets wether the user is allowed to drag the value outside the range.", 
   widget, 
   fltk_value_output, 
   rval, 
   int)
{
  GFL_ASSERT_1(fltk_value_output_get_soft, widget);  
  Fl_Value_Output *wi = (Fl_Value_Output *) SCM_CDR (widget);
  return gh_int2scm((uchar)wi->soft());
}


GFL_PROCEDURE_2_0
  ( fltk_value_output_set_soft, 
   "fltk-value-output::set-soft", 
   "Sets wether the user is allowed to drag the value outside the range.", 
   widget, a, 
   fltk_value_output, int)
{
  GFL_ASSERT_2(fltk_value_output_set_soft, widget, a);
  Fl_Value_Output *wi = (Fl_Value_Output *) SCM_CDR (widget);
  wi->soft((uchar)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_value_output_init ()
{
  GFL_NEW_TYPE (fltk_value_output);
  GFL_NEW_PROCEDURE (fltk_value_output_new);
  GFL_NEW_PROCEDURE (fltk_value_output_get_soft);
  GFL_NEW_PROCEDURE (fltk_value_output_set_soft);
}

