/*
  Copyright 1999 by James Dean Palmer.
  
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Window.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Widget *) gh_cdr(widget_smob);
  gfl_remove(widget_smob);
  return (sizeof (GFL_Widget) + sizeof (Fl_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Widget *widget = (Fl_Widget *) gh_cdr(widget_smob);
  scm_puts ("#<widget Fl_Widget ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_CLASS_TYPE ("fltk-widget", 
		"fltk-widget is the base class for all fltk widgets.",
		fltk_widget, 
		Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_1_0
( fltk_widget_delete, 
  "fltk-widget::delete", 
  "Deletes the widget.", 
  widget, 
  fltk_widget)
{
  GFL_ASSERT_1(fltk_widget_delete, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  gfl_remove (widget);
  delete wi;
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_x, 
  "fltk-widget::x", 
  "Gets the x position.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_x, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm(wi->x());
}

GFL_PROCEDURE_1_1
( fltk_widget_y, 
  "fltk-widget::y", 
  "Gets the y position.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_y, widget);  
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm(wi->y());
}

GFL_PROCEDURE_1_1
( fltk_widget_w, 
  "fltk-widget::w", 
  "Gets the width.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_w, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm(wi->w());
}

GFL_PROCEDURE_1_1
( fltk_widget_h, 
  "fltk-widget::h", 
  "Gets the height.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_h, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm(wi->h());
}

GFL_PROCEDURE_5_0
( fltk_widget_resize, 
  "fltk-widget::resize", 
  "Resize the widget to the given size and position.", 
  widget, x, y, w, h, 
  fltk_widget, int, int, int, int)
{
  GFL_ASSERT_5(fltk_widget_resize, widget, x, y, w, h);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->resize(gh_scm2int(x), gh_scm2int(y), 
	     gh_scm2int(w), gh_scm2int(h));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_widget_position, 
  "fltk-widget::position", 
  "Sets the position.", 
  widget, x, y, 
  fltk_widget, int, int)
{
  GFL_ASSERT_3(fltk_widget_position, widget, x, y);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->position(gh_scm2int(x), gh_scm2int(y));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_widget_size, 
  "fltk-widget::size", 
  "Sets the size.", 
  widget, w, h, 
  fltk_widget, int, int)
{
  GFL_ASSERT_3(fltk_widget_size, widget, w, h);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->size(gh_scm2int(w), gh_scm2int(h));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_window, 
  "fltk-widget::window", 
  "Returns the fltk-window this widget is in.", 
  widget, 
  fltk_widget, 
  rval, 
  fltk_widget)
{
  GFL_ASSERT_1(fltk_widget_window, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gfl_lookup((Fl_Widget *) wi->window());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_box, 
  "fltk-widget::set-box", 
  "Sets the widget box type.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_box, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->box((Fl_Boxtype)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_box, 
  "fltk-widget::get-box", 
  "Gets the widget box type.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_box, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->box());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_type, 
  "fltk-widget::set-type", 
  "Sets the widget type.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_type, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->type((uchar)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_type, 
  "fltk-widget::get-type", 
  "Gets the widget type.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_type, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->type());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_color, 
  "fltk-widget::set-color", 
  "Sets the widget color.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_color, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->color(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_color, 
  "fltk-widget::get-color", 
  "Gets the widget color.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_color, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->color());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_selection_color, 
  "fltk-widget::set-selection-color", 
  "Sets the selection color.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_selection_color, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->selection_color(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_selection_color, 
  "fltk-widget::get-selection-color", 
  "Gets the selection color.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_selection_color, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->selection_color());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_label, 
  "fltk-widget::set-label", 
  "Sets the widget label.", 
  widget, a, 
  fltk_widget, str)
{
  GFL_ASSERT_2(fltk_widget_set_label, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  if (wi->label() != NULL)
    free((char *)wi->label());
  wi->label(gh_scm2newstr(a, NULL));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_widget_get_label, 
  "fltk-widget::get-label", 
  "Gets the widget label.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_get_label, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_str02scm((char *)wi->label());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_label_type, 
  "fltk-widget::set-label-type", 
  "Sets the widget label type.", 
  widget, a, 
  fltk_widget, int)  
{
  GFL_ASSERT_2(fltk_widget_set_label_type, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->labeltype((Fl_Labeltype)gh_scm2long(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_label_type, 
  "fltk-widget::get-label-type", 
  "Gets the widget label type.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_label_type, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->labeltype());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_align, 
  "fltk-widget::set-align", 
  "Sets the widget alignment.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_align, widget, a);  
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->align(gh_scm2long(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_align, 
  "fltk-widget::get-align", 
  "Gets the widget alignment.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_align, widget); 
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->align());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_label_color, 
  "fltk-widget::set-label-color", 
  "Sets the widget label color.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_label_color, widget, a);  
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->labelcolor(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_label_color, 
  "fltk-widget::get-label-color", 
  "Gets the widget label color.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_label_color, widget);  
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->labelcolor());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_label_font, 
  "fltk-widget::set-label-font", 
  "Sets the label font.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_label_font, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->labelfont(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_label_font, 
  "fltk-widget::get-label-font", 
  "Gets the label font.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_label_font, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->labelfont());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_label_size, 
  "fltk-widget::set-label-size", 
  "Sets the label font size.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_label_size, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->labelsize(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_label_size, 
  "fltk-widget::get-label-size", 
  "Gets the label font size.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_label_size, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->labelsize());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_callback, 
  "fltk-widget::set-callback", 
  "Sets the callback.", 
  widget, a, 
  fltk_widget, callback)
{
  GFL_ASSERT_2(fltk_widget_set_callback, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  gfl_set_callback(wi, a);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_callback, 
  "fltk-widget::get-callback", 
  "Gets the callback.", 
  widget, 
  fltk_widget, 
  rval, 
  fltk_widget)
{
  GFL_ASSERT_1(fltk_widget_get_callback, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gfl_get_callback(wi);
}

GFL_PROCEDURE_2_0
( fltk_widget_set_user_data, 
  "fltk-widget::set-user-data", 
  "Sets the user data.", 
  widget, sb, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_user_data, widget, sb);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->user_data((void *)sb);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_widget_get_user_data, 
  "fltk-widget::get-user-data", 
  "Gets the user data.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_get_user_data, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return ((SCM)wi->user_data());
}

GFL_PROCEDURE_2_0
( fltk_widget_set_argument, 
  "fltk-widget::set-argument", 
  "Sets the callback argument.", 
  widget, argument, 
  fltk_widget, any)
{
  GFL_ASSERT_2(fltk_widget_set_argument, widget, argument);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->argument((long)argument);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_argument, 
  "fltk-widget::get-argument", 
  "docstring", 
  widget, 
  fltk_widget,
  argument,
  any)
{
  GFL_ASSERT_1(fltk_widget_get_argument, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return ((SCM)wi->argument());
}

GFL_PROCEDURE_1_0
( fltk_widget_set_changed, 
  "fltk-widget::set-changed", 
  "Sets wether the widget has changed.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_set_changed, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->set_changed();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_changed, 
  "fltk-widget::get-changed", 
  "Gets wether the widget has changed.", 
  widget, 
  fltk_widget, 
  rval, 
  bool)
{
  GFL_ASSERT_1(fltk_widget_get_changed, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  if (wi->changed())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}


GFL_PROCEDURE_1_0
( fltk_widget_clear_changed, 
  "fltk-widget::clear-changed", 
  "Sets the widget as unchanged.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_clear_changed, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->clear_changed();
  return SCM_UNSPECIFIED;
}


GFL_PROCEDURE_2_0
( fltk_widget_set_when, 
  "fltk-widget::set-when", 
  "Sets when the callback occurs.", 
  widget, a, 
  fltk_widget, int)
{
  GFL_ASSERT_2(fltk_widget_set_when, widget, a);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->when(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_get_when, 
  "fltk-widget::get-when", 
  "Gets when the callback occurs.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_get_when, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->when());
}

GFL_PROCEDURE_1_1
( fltk_widget_visible, 
  "fltk-widget::visible", 
  "Sets the widget as visible.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_visible, widget);
  
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((long)wi->visible());
}

GFL_PROCEDURE_1_0
( fltk_widget_show, 
  "fltk-widget::show", 
  "Shows the widget.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_show, widget);  
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->show();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_widget_hide, 
  "fltk-widget::hide", 
  "Hides the widget.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_hide, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->hide();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_active, 
  "fltk-widget::active", 
  "Gets wether the widget is active.", 
  widget, 
  fltk_widget, 
  rval, 
  bool)
{
  GFL_ASSERT_1(fltk_widget_active, widget);  
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  if (wi->active())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_0
( fltk_widget_activate, 
  "fltk-widget::activate", 
  "Activates the widget.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_activate, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->activate();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_widget_deactivate, 
  "fltk-widget::deactivate", 
  "Deactivates the widget.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_deactivate, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->deactivate();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_widget_redraw, 
  "fltk-widget::redraw", 
  "Redraw the widget.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_redraw, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->redraw();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_widget_damage, 
  "fltk-widget::damage", 
  "Damage the widget.", 
  widget, 
  fltk_widget, 
  rval, 
  long)
{
  GFL_ASSERT_1(fltk_widget_damage, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gh_long2scm((int)wi->damage());
}

GFL_PROCEDURE_1_1
( fltk_widget_parent, 
  "fltk-widget::parent", 
  "Gets the parent widget.", 
  widget, 
  fltk_widget, 
  rval, 
  fltk_widget)
{
  GFL_ASSERT_1(fltk_widget_parent, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  return gfl_lookup((Fl_Widget *) wi->parent());
}

GFL_PROCEDURE_2_1
( fltk_widget_inside, 
  "fltk-widget::inside", 
  "Returns if there is a child widget.", 
  widget, widget2, 
  fltk_widget, fltk_widget, 
  rval, 
  bool)
{
  GFL_ASSERT_2(fltk_widget_inside, widget, widget2);
  Fl_Widget *wi2;
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi2 = (Fl_Widget *) gh_cdr(widget2);
  if (wi->inside(wi2))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_1
( fltk_widget_contains, 
  "fltk-widget::contains", 
  "Returns if this widget contains the given child widget.", 
  widget, widget_2, 
  fltk_widget, fltk_widget, 
  rval, 
  bool)
{
  GFL_ASSERT_2(fltk_widget_contains, widget, widget_2);
  Fl_Widget *wi_2;
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi_2 = (Fl_Widget *) gh_cdr(widget_2);
  if (wi->contains(wi_2))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_0
( fltk_widget_take_focus, 
  "fltk-widget::take-focus", 
  "This widget will take the focus.", 
  widget, 
 fltk_widget )
{
  GFL_ASSERT_1(fltk_widget_take_focus, widget);
  Fl_Widget *wi = (Fl_Widget *) gh_cdr(widget);
  wi->take_focus();
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_widget_init ()
{
  GFL_NEW_TYPE (fltk_widget);
  GFL_NEW_PROCEDURE (fltk_widget_delete);
  GFL_NEW_PROCEDURE (fltk_widget_x);
  GFL_NEW_PROCEDURE (fltk_widget_y);
  GFL_NEW_PROCEDURE (fltk_widget_w);
  GFL_NEW_PROCEDURE (fltk_widget_h);
  GFL_NEW_PROCEDURE (fltk_widget_resize);
  GFL_NEW_PROCEDURE (fltk_widget_position);
  GFL_NEW_PROCEDURE (fltk_widget_size);
  GFL_NEW_PROCEDURE (fltk_widget_window);
  GFL_NEW_PROCEDURE (fltk_widget_set_box);
  GFL_NEW_PROCEDURE (fltk_widget_get_box);
  GFL_NEW_PROCEDURE (fltk_widget_set_type);
  GFL_NEW_PROCEDURE (fltk_widget_get_type);
  GFL_NEW_PROCEDURE (fltk_widget_set_color);
  GFL_NEW_PROCEDURE (fltk_widget_get_color);
  GFL_NEW_PROCEDURE (fltk_widget_set_selection_color);
  GFL_NEW_PROCEDURE (fltk_widget_get_selection_color);
  GFL_NEW_PROCEDURE (fltk_widget_set_label);
  GFL_NEW_PROCEDURE (fltk_widget_get_label);
  GFL_NEW_PROCEDURE (fltk_widget_set_label_type);
  GFL_NEW_PROCEDURE (fltk_widget_get_label_type);
  GFL_NEW_PROCEDURE (fltk_widget_set_align);
  GFL_NEW_PROCEDURE (fltk_widget_get_align);
  GFL_NEW_PROCEDURE (fltk_widget_set_label_color);
  GFL_NEW_PROCEDURE (fltk_widget_get_label_color);
  GFL_NEW_PROCEDURE (fltk_widget_set_label_font);
  GFL_NEW_PROCEDURE (fltk_widget_get_label_font);
  GFL_NEW_PROCEDURE (fltk_widget_set_label_size);
  GFL_NEW_PROCEDURE (fltk_widget_get_label_size);
  GFL_NEW_PROCEDURE (fltk_widget_set_callback);
  GFL_NEW_PROCEDURE (fltk_widget_get_callback);
  GFL_NEW_PROCEDURE (fltk_widget_set_user_data);
  GFL_NEW_PROCEDURE (fltk_widget_get_user_data);
  GFL_NEW_PROCEDURE (fltk_widget_set_argument);
  GFL_NEW_PROCEDURE (fltk_widget_get_argument);
  GFL_NEW_PROCEDURE (fltk_widget_set_changed);
  GFL_NEW_PROCEDURE (fltk_widget_get_changed);
  GFL_NEW_PROCEDURE (fltk_widget_clear_changed);
  GFL_NEW_PROCEDURE (fltk_widget_set_when);
  GFL_NEW_PROCEDURE (fltk_widget_get_when);
  GFL_NEW_PROCEDURE (fltk_widget_visible);
  GFL_NEW_PROCEDURE (fltk_widget_show);
  GFL_NEW_PROCEDURE (fltk_widget_hide);
  GFL_NEW_PROCEDURE (fltk_widget_active);
  GFL_NEW_PROCEDURE (fltk_widget_activate);
  GFL_NEW_PROCEDURE (fltk_widget_deactivate);
  GFL_NEW_PROCEDURE (fltk_widget_redraw);
  GFL_NEW_PROCEDURE (fltk_widget_damage);
  GFL_NEW_PROCEDURE (fltk_widget_parent);
  GFL_NEW_PROCEDURE (fltk_widget_inside);
  GFL_NEW_PROCEDURE (fltk_widget_contains);
  GFL_NEW_PROCEDURE (fltk_widget_take_focus);
}
