/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Window.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Window *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (GFL_Widget) + sizeof (Fl_Window));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Window *widget = (Fl_Window *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Window ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-window", 
		   "fltk-window produces an actual window.",
		   fltk_window, fltk_group, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_window_new,
  "fltk-window::new",
  "Creates a new fltk-window with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  window,
  fltk_window )
{
  GFL_ASSERT_5 (fltk_window_new, x, y, w, h, label);
  Fl_Window * widget = new Fl_Window(gh_scm2int(x), gh_scm2int(y), 
				     gh_scm2int(w), gh_scm2int(h),
				     gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_window, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_0
( fltk_window_show, 
  "fltk-window::show", 
  "Show the window.", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_show, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->show();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_window_hide, 
  "fltk-window::hide", 
  "Hide the window.", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_hide, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->hide();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_window_shown, 
  "fltk-window::shown", 
  "Returns wether the window is being shown.", 
  widget, 
  fltk_window, 
  rval, 
  bool)
{
  GFL_ASSERT_1(fltk_window_shown, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  if (wi->shown())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_0
( fltk_window_iconize, 
  "fltk-window::iconize", 
  "Iconize the window.", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_iconize, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->iconize();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_5_0
( fltk_window_resize, 
  "fltk-window::resize", 
  "Resize the window with the given position and size.", 
  widget, x, y, w, h, 
  fltk_window, int, int, int, int)
{
  GFL_ASSERT_5(fltk_window_resize, widget, x, y, w, h);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->resize(gh_scm2int(x), gh_scm2int(y), gh_scm2int(w), gh_scm2int(h));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_window_position, 
  "fltk-window::position", 
  "Sets the window position.", 
  widget, x, y, 
  fltk_window, int, int)
{
  GFL_ASSERT_3(fltk_window_position, widget, x, y);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->position(gh_scm2int(x), gh_scm2int(y));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_window_size, 
  "fltk-window::size", 
  "docstring", 
  widget, w, h, 
  fltk_window, int, int)
{
  GFL_ASSERT_3(fltk_window_size, widget, w, h);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->size(gh_scm2int(w), gh_scm2int(h));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_window_free_position, 
  "fltk-window::free-position", 
  "Sets the window position as \"free\".", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_free_position, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->free_position();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_4_0
( fltk_window_hotspot, 
  "fltk-window::hotspot", 
  "Sets the window hotspot.", 
  widget, x, y, off, 
  fltk_window, int, int, int)
{
  GFL_ASSERT_4(fltk_window_hotspot, widget, x, y, off);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->hotspot(gh_scm2int(x), gh_scm2int(y), gh_scm2int(off));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_window_fullscreen, 
  "fltk-window::fullscreen", 
  "Puts the window in fullscreen mode.", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_fullscreen, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->fullscreen();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_5_0
( fltk_window_fullscreen_off, 
  "fltk-window::fullscreen-off", 
  "Turns off fullscreen mode.", 
  widget, x, y, w, h, 
  fltk_window, int, int, int, int)
{
  GFL_ASSERT_5(fltk_window_fullscreen_off, widget, x, y, w, h);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->fullscreen_off(gh_scm2int(x), gh_scm2int(y), 
		     gh_scm2int(w), gh_scm2int(h));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_window_set_border, 
  "fltk-window::set-border", 
  "Sets the border.", 
  widget, x, 
  fltk_window, int)
{
  GFL_ASSERT_2(fltk_window_set_border, widget, x);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->border(SCM_NFALSEP(x));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_window_get_border, 
  "fltk-window::get-border", 
  "Gets the border.", 
  widget, 
  fltk_window, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_window_get_border, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  return gh_int2scm((int)wi->border());
}

GFL_PROCEDURE_1_0
( fltk_window_clear_border, 
  "fltk-window::clear-border", 
  "Clear the border.", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_clear_border, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->clear_border();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_window_set_modal, 
  "fltk-window::set-modal", 
  "Set the window as modal.", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_set_modal, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->set_modal();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_window_get_modal, 
  "fltk-window::get-modal", 
  "Gets wether the window is modal.", 
  widget, 
  fltk_window, 
  rval, 
  bool)
{
  GFL_ASSERT_1(fltk_window_get_modal, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  if (wi->modal())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_0
( fltk_window_set_non_modal, 
  "fltk-window::set-non-modal", 
  "docstring", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_set_non_modal, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->set_non_modal();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_window_get_non_modal, 
  "fltk-window::get-non-modal", 
  "docstring", 
  widget, 
  fltk_window, 
  rval, 
  bool)
{
  GFL_ASSERT_1(fltk_window_get_non_modal, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  if (wi->non_modal())
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
( fltk_window_set_label, 
  "fltk-window::set-label", 
  "docstring", 
  widget, label, 
  fltk_window, str)
{
  GFL_ASSERT_2(fltk_window_set_label, widget, label);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  free((char *)wi->label());
  wi->label(gh_scm2newstr(label, NULL));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_window_get_label, 
  "fltk-window::get-label", 
  "docstring", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_get_label, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  return scm_makfrom0str (wi->label());
}

GFL_PROCEDURE_2_0
( fltk_window_set_iconlabel, 
  "fltk-window::set-iconlabel", 
  "docstring", 
  widget, label, 
  fltk_window, str)
{
  GFL_ASSERT_2(fltk_window_set_iconlabel, widget, label);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  if (wi->iconlabel() != NULL) 
    free((char *)wi->iconlabel());
  wi->iconlabel(gh_scm2newstr(label, NULL));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_window_get_iconlabel, 
  "fltk-window::get-iconlabel", 
  "docstring", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_get_iconlabel, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  return scm_makfrom0str (wi->iconlabel());
}

GFL_PROCEDURE_2_0
( fltk_window_set_xclass, 
  "fltk-window::set-xclass", 
  "docstring", 
  widget, label, 
  fltk_window, str)
{
  GFL_ASSERT_2(fltk_window_set_xclass, widget, label);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->xclass(gh_scm2newstr(label, NULL));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_window_get_xclass, 
  "fltk-window::get-xclass", 
  "docstring", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_get_xclass, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  return scm_makfrom0str (wi->xclass());
}

GFL_PROCEDURE_1_0
( fltk_window_make_current, 
  "fltk-window::make-current", 
  "docstring", 
  widget, 
 fltk_window )
{
  GFL_ASSERT_1(fltk_window_make_current, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  wi->make_current();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_window_current, 
  "fltk-window::current", 
  "docstring", 
  widget, 
  fltk_window, 
  rval, 
  fltk_window)
{
  GFL_ASSERT_1(fltk_window_current, widget);
  Fl_Window *wi = (Fl_Window *) gh_cdr (widget);
  return gfl_lookup((Fl_Widget *) wi->current());
}

/*
   4.  Initialization Hook:
*/

void
fltk_window_init ()
{
  GFL_NEW_TYPE (fltk_window);
  GFL_NEW_PROCEDURE (fltk_window_new);
  //gh_eval_str("(define* (fltk-window-new label "
  //"#&key (x -1) (y -1) (w 100) (h 100)) "
  //"(fltk-window-new* x y w h label))");
  GFL_NEW_PROCEDURE (fltk_window_show);
  GFL_NEW_PROCEDURE (fltk_window_hide);
  GFL_NEW_PROCEDURE (fltk_window_shown);
  GFL_NEW_PROCEDURE (fltk_window_iconize);
  GFL_NEW_PROCEDURE (fltk_window_resize);
  GFL_NEW_PROCEDURE (fltk_window_free_position);
  GFL_NEW_PROCEDURE (fltk_window_hotspot);
  GFL_NEW_PROCEDURE (fltk_window_fullscreen);
  GFL_NEW_PROCEDURE (fltk_window_fullscreen_off);
  GFL_NEW_PROCEDURE (fltk_window_set_border);
  GFL_NEW_PROCEDURE (fltk_window_get_border);
  GFL_NEW_PROCEDURE (fltk_window_clear_border);
  GFL_NEW_PROCEDURE (fltk_window_set_modal);
  GFL_NEW_PROCEDURE (fltk_window_get_modal);
  GFL_NEW_PROCEDURE (fltk_window_set_non_modal);
  GFL_NEW_PROCEDURE (fltk_window_get_non_modal);
  GFL_NEW_PROCEDURE (fltk_window_set_label);
  GFL_NEW_PROCEDURE (fltk_window_get_label);
  GFL_NEW_PROCEDURE (fltk_window_set_iconlabel);
  GFL_NEW_PROCEDURE (fltk_window_get_iconlabel);
  GFL_NEW_PROCEDURE (fltk_window_set_xclass);
  GFL_NEW_PROCEDURE (fltk_window_get_xclass);
  GFL_NEW_PROCEDURE (fltk_window_make_current);
  GFL_NEW_PROCEDURE (fltk_window_current);
  GFL_NEW_PROCEDURE (fltk_window_size);
  GFL_NEW_PROCEDURE (fltk_window_position);
}

