/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Widget.H>
#include <gFL/guile_fltk.h>

GFL_Widget *gfl_widget_db = 0;
static int gfl_widget_db_size = 0;

GFL_TYPE ("float", "A floating point value.", float);
GFL_TYPE ("long", "A long value.", long);
GFL_TYPE ("int", "A int value.", int);
GFL_TYPE ("char", "A char value.", char);
GFL_TYPE ("bool", "A bool value.", bool);
GFL_TYPE ("bool", "A bool value.", double);
GFL_TYPE ("str", "A str value.", str);
GFL_TYPE ("symbol", "A symbol value.", symbol);
GFL_TYPE ("int_array", "A str value.", int_array);
GFL_TYPE ("callback", "callback doc", callback);
GFL_TYPE ("fltk-boxtype", "fltk-boxtype doc", fltk_boxtype);
GFL_TYPE ("fltk-color", "fltk-color doc", fltk_color);
GFL_TYPE ("fltk-font", "fltk-font doc", fltk_font);
GFL_TYPE ("fltk-when", "fltk-when doc", fltk_when);
GFL_TYPE ("fltk-menu-item", "fltk-mwnu-item doc", fltk_menu_item);
GFL_TYPE ("fltk-buttontype", "fltk-buttontype doc", fltk_menu_buttontype);
GFL_TYPE ("fltk-charttype", "fltk-charttype doc", fltk_charttype);
GFL_TYPE ("any", "Any value.", any);

/*

   The object_registry is a vector of all the widget smobs allocated from 
   the Guile interface.  We can register new objects with 
   gfltk_register(), and we can lookup a widget smob (given the widget 
   pointer) with gfltk_lookup()

*/

static void
gfl_append ()
{
  gfl_widget_db_size++;
  gfl_widget_db = (GFL_Widget *) realloc (gfl_widget_db, sizeof(GFL_Widget) * gfl_widget_db_size);
  return;
}

void 
gfl_register (SCM b)
{
  GFL_Widget *N; 
  gfl_append();
  N = &gfl_widget_db[gfl_widget_db_size-1];
  N->smob = b;
  N->widget = (Fl_Widget *) SCM_CDR (b);
  N->callback = SCM_BOOL_F;
  scm_protect_object (N->callback);
  scm_protect_object (N->smob);
}

SCM 
gfl_register_as (Fl_Widget *widget, long tag)
{
  SCM widget_smob;
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, tag);
  gfl_register (widget_smob);
  return widget_smob;
}

void 
gfl_remove (SCM a)
{
  int i, j;
  // printf("REMOVING SCHEME OBJECT!!!\n");
  for (i = 0; i < gfl_widget_db_size; i++)
    {
      if (gfl_widget_db[i].smob == a)
	{
	  gfl_widget_db_size--;
	  for (j = i; j < gfl_widget_db_size; j++)
	    gfl_widget_db[j] = gfl_widget_db[j+1];	  
	  realloc (gfl_widget_db, sizeof(GFL_Widget) * (gfl_widget_db_size+1));
	  return;
	}
    }
}

SCM 
gfl_lookup(Fl_Widget *a)
{
  int i;
  for (i = 0; i < gfl_widget_db_size; i++)
    {
      if (gfl_widget_db[i].widget == a)
	return gfl_widget_db[i].smob;
    }
  return SCM_BOOL_F;
}

SCM 
gfl_lookup2(Fl_Widget *a, long tag)
{
  int i;
  for (i = 0; i < gfl_widget_db_size; i++)
    {
      if (gfl_widget_db[i].widget == a)
	return gfl_widget_db[i].smob;
    }
  return gfl_register_as(a, tag);
}

void 
gfl_set_callback(Fl_Widget *widget, SCM callback)
{
  int i;
  for (i = 0; i < gfl_widget_db_size; i++)
    {
      if (gfl_widget_db[i].widget == widget)
	{
	  widget->callback (guile_fltk_callback);
	  widget->argument (callback);
	  // scm_unprotect_object (gfl_widget_db[i]->callback);
	  gfl_widget_db[i].callback = callback;
	  scm_protect_object (callback);
	  return;
	}
    }
}

static int
gfl_get_index(Fl_Widget *widget)
{
  int i;
  for (i = 0; i < gfl_widget_db_size; i++)
    if (gfl_widget_db[i].widget == widget)
      return i;  
  return -1;
}

SCM
gfl_get_callback(Fl_Widget *widget)
{
  int i = gfl_get_index (widget);
  if (i >= 0)
    return gfl_widget_db[i].callback;
  return SCM_BOOL_F;
}

void 
guile_fltk_callback (Fl_Widget *a, void *b)
{
  int i = gfl_get_index (a);
  if (i < 0)
    return;
  SCM callback = gfl_widget_db[i].callback;
  if (callback == SCM_BOOL_F)
    return;

  if (SCM_CLOSUREP ((SCM) callback))
    gh_call2 ((SCM) callback, (SCM) gfl_widget_db[i].smob, (SCM) a->user_data ());
}

void 
guile_fltk_callback2 (Fl_Widget *a, void *b)
{
  SCM aa = (SCM) b;

  if (SCM_CLOSUREP ((SCM) aa))
    gh_call0 ((SCM) aa);
}

int 
gfl_type_p (SCM a, long b)
{
  /*
  int i;
  long aa = gh_car (a);
  if (!SCM_NIMP (a))
    return 0;

  if (aa == b) return 1;
  for (i = 0; i < class_registry.size (); i++)
    {
      if (class_registry[i]->a == aa)
	if (class_registry[i]->b == b)
	  return 1;
    }
  return 0;
  */
  return 1;
}

/*
   gfltk_export_widget is a function specifically designed to export
   c++ initiated widgets to the Guile object name space.
*/

void
gfl_export_object (char *name, Fl_Widget *widget, long widget_tag)
{
  SCM widget_smob;
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, widget_tag);
  gfl_register (widget_smob);
  gh_define (name, widget_smob);
}

/*
   When the fltk module is initially loaded we need to register each
   fltk data type and define the fltk scheme procedures.
*/

void
scm_init_fltk ()
{
  int verbose = 0;
  gfl_init ();

  GFL_NEW_TYPE (float);
  GFL_NEW_TYPE (long);
  GFL_NEW_TYPE (int);
  GFL_NEW_TYPE (char);
  GFL_NEW_TYPE (bool);
  GFL_NEW_TYPE (double);
  GFL_NEW_TYPE (str);
  GFL_NEW_TYPE (symbol);
  GFL_NEW_TYPE (any);
  GFL_NEW_TYPE (int_array);
  GFL_NEW_TYPE (callback);
  GFL_NEW_TYPE (fltk_boxtype);
  GFL_NEW_TYPE (fltk_color);
  GFL_NEW_TYPE (fltk_font);
  GFL_NEW_TYPE (fltk_when);
  GFL_NEW_TYPE (fltk_menu_item);
  GFL_NEW_TYPE (fltk_menu_buttontype);
  GFL_NEW_TYPE (fltk_charttype);
  
  if (verbose) printf ("Initializing Fl..\n");
  fltk_fl_init ();
  if (verbose) printf ("Initializing Widget..\n");
  fltk_widget_init ();
  if (verbose) printf ("Initializing Group..\n");
  fltk_group_init ();
  if (verbose) printf ("Initializing Tile..\n");
  fltk_tile_init ();
  if (verbose) printf ("Initializing Menu..\n");
  fltk_menu_init ();
  if (verbose) printf ("Initializing Menu_Bar..\n");
  fltk_menu_bar_init ();
  if (verbose) printf ("Initializing Browser..\n");
  fltk_browser_base_init ();
  if (verbose) printf ("Initializing Browser Base..\n");
  fltk_browser_init ();
  if (verbose) printf ("Initializing Hold Browser..\n");
  fltk_hold_browser_init ();
  if (verbose) printf ("Initializing Multi Browser..\n");
  fltk_multi_browser_init ();
  if (verbose) printf ("Initializing Select Browser..\n");
  fltk_select_browser_init ();
  if (verbose) printf ("Initializing Window..\n");
  fltk_window_init ();
  if (verbose) printf ("Initializing Button..\n");
  fltk_button_init ();
  if (verbose) printf ("Initializing Box..\n");
  fltk_box_init ();
  if (verbose) printf ("Initializing Check Button..\n");
  fltk_check_button_init ();
  if (verbose) printf ("Initializing Light Button..\n");
  fltk_light_button_init ();
  if (verbose) printf ("Initializing Repeat Button..\n");
  fltk_repeat_button_init ();
  if (verbose) printf ("Initializing Return Button..\n");
  fltk_return_button_init ();
  if (verbose) printf ("Initializing Round Button..\n");
  fltk_round_button_init ();
  if (verbose) printf ("Initializing Valuator..\n");
  fltk_valuator_init ();
  if (verbose) printf ("Initializing Adjuster..\n");
  fltk_adjuster_init ();
  if (verbose) printf ("Initializing Counter..\n");
  fltk_counter_init ();
  if (verbose) printf ("Initializing Dial..\n");
  fltk_dial_init ();
  if (verbose) printf ("Initializing Roller..\n");
  fltk_roller_init ();
  if (verbose) printf ("Initializing Slider..\n");
  fltk_slider_init ();
  if (verbose) printf ("Initializing Value Input..\n");
  fltk_value_input_init ();
  if (verbose) printf ("Initializing Value Output..\n");
  fltk_value_output_init ();
  if (verbose) printf ("Initializing Input_ (base)..\n");
  fltk_input_base_init ();
  if (verbose) printf ("Initializing Input..\n");
  fltk_input_init ();
  if (verbose) printf ("Initializing Float Input..\n");
  fltk_float_input_init ();
  if (verbose) printf ("Initializing Int Input..\n");
  fltk_int_input_init ();
  if (verbose) printf ("Initializing Secret Input..\n");
  fltk_secret_input_init ();
  if (verbose) printf ("Initializing Multiline Input..\n");
  fltk_multiline_input_init ();
  if (verbose) printf ("Initializing Chart..\n");
  fltk_chart_init ();
  if (verbose) printf ("Initializing Choice..\n");
  fltk_choice_init ();
  if (verbose) printf ("Initializing Color Chooser..\n");
  fltk_color_chooser_init ();
  if (verbose) printf ("Initializing Clock..\n");
  fltk_clock_init ();
  if (verbose) printf ("Initializing Round Clock..\n");
  fltk_round_clock_init ();
  if (verbose) printf ("Initializing Menu Button..\n");
  fltk_menu_button_init ();
  if (verbose) printf ("Initializing Fltk Timer..\n");
  fltk_timer_init ();
  if (verbose) printf ("Initializing Fltk Value Slider..\n");
  fltk_value_slider_init ();
  if (verbose) printf ("Initializing Fltk Tabs..\n");
  fltk_tabs_init ();
  if (verbose) printf ("Completed Fltk Init..\n");

  scm_gc ();
}

/*
   This provides the "module hook" into the shared library.
*/

void
scm_init_gfltk_gfltk_module ()
{
  scm_register_module_xxx ("gfltk gfltk", (void*) scm_init_fltk);
}


