#include <gFL/zAVLTree.h>
#include <gFL/gfl.h>

typedef struct Gfl_Record
{
  Gfl_Info *info;
  Gfl_Proc proc;
  long arg;
} Gfl_Record;

static zAVLTree   *gfl;
static Gfl_Record *gfl_last_record = 0;
static Gfl_Record *gfl_current_record = 0;

static const char* 
gfl_get_key (void const *item)
{
  if (item)
    return ((Gfl_Record *) item)->info[0].label;
  return 0;
}

Gfl_Record *
gfl_find_record (const char *name)
{
  Gfl_Record *result;

  /* Short circuit if possible. */  
  if (gfl->count == 0) return 0;

  if (gfl_last_record && 
      (!strcmp (name, gfl_get_key ((const void *)gfl_last_record))))
    return (gfl_last_record);

  result = (Gfl_Record *) zAVLSearch (gfl, name);
  gfl_last_record = result;

  if (result)
    return result;  

  printf ("WARNING: Couldn't find a procedural database entry for \"%s\".\n", 
	  name);
  return 0;
}

Gfl_Info*
gfl_find_info (char *name)
{
  Gfl_Record *t;
  t = gfl_find_record (name);
  if (t)
    return t->info;
  return 0;
}

SCM
gfl_exe (char *name, ...)
{
  SCM rvalue;
  va_list arg;
  
  va_start (arg, name);
  if ((gfl_current_record = gfl_find_record (name)))
    {
      /* printf("GFL:Found record..\n"); */
      rvalue = gfl_current_record->proc (arg);
    }
  else rvalue = SCM_BOOL_F;
  
  va_end (arg);
  return rvalue;
}

int 
gfl_add_record (Gfl_Record record)
{
  Gfl_Record *a;

  a = (Gfl_Record *) malloc (sizeof (Gfl_Record));
  *a = record;
  
  return zAVLInsert (gfl, (void *)a);
}

Gfl_Class * gfl_find_class (long);

void 
gfl_assert (Gfl_Info *args, ...)
{
  int i;
  Gfl_Info *t;
  SCM x;  
  va_list ap = 0;
  if (args[2].id !=0)
    va_start (ap, args);
  for (i = 2, t = &args[i]; (t->id != 0); i++, t = &args[i])
    {
      x = va_arg (ap, SCM);
      if ((t->id) == ((long)&double_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&float_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&long_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&int_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&str_class_tag)) {
	SCM_ASSERT (gh_string_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&symbol_class_tag)) {
	SCM_ASSERT (gh_symbol_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&bool_class_tag)) {
	SCM_ASSERT (gh_boolean_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&char_class_tag)) {
	SCM_ASSERT (gh_char_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&int_array_class_tag)) {
	SCM_ASSERT (gh_vector_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&callback_class_tag)) {
	SCM_ASSERT ((x==SCM_BOOL_F || gh_procedure_p (x)), x, 
		    ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&fltk_font_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&fltk_boxtype_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&fltk_menu_buttontype_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&fltk_charttype_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&fltk_when_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if ((t->id) == ((long)&fltk_color_class_tag)) {
	SCM_ASSERT (gh_number_p (x), x, ((i < 8) ? (i - 1) : 0), 
		    args[0].label); continue; }
      if (SCM_NIMP(x))
	{
	  long a;
	  Gfl_Class * b;
	  int c, i;
	  a = gh_car (x);	      
	  b = gfl_find_class (a);
	  c = 0;
	  if (b->tag == a) c = 1;
	  for (i = 0; i<b->subclasses_cnt; i++)
	    if (b->subclasses[i] == a) c = 1;
	  SCM_ASSERT (c, x, ((i < 8) ? (i - 1) : 0), 
		      args[0].label);
	}
      else
	{
	  printf("WARNING: Type has slipped through the cracks.\n");
	  SCM_ASSERT (0, x, ((i < 8) ? (i - 1) : 0),
		      args[0].label);
	}
    }
  if (args[2].id !=0)
    va_end (ap);
}

int 
gfl_add_ (Gfl_Info *info, Gfl_Proc proc, long arg)
{
  Gfl_Record t;
  t.info = info;
  t.proc = proc;
  t.arg = arg;
  return gfl_add_record (t);
}

int gfl_add0 (Gfl_Info *info, Gfl_Proc0 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add1 (Gfl_Info *info, Gfl_Proc1 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add2 (Gfl_Info *info, Gfl_Proc2 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add3 (Gfl_Info *info, Gfl_Proc3 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add4 (Gfl_Info *info, Gfl_Proc4 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add5 (Gfl_Info *info, Gfl_Proc5 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add6 (Gfl_Info *info, Gfl_Proc6 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add7 (Gfl_Info *info, Gfl_Proc7 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add8 (Gfl_Info *info, Gfl_Proc8 proc, long argument)
  { return gfl_add_ (info, proc, argument); }
int gfl_add9 (Gfl_Info *info, Gfl_Proc9 proc, long argument)
  { return gfl_add_ (info, proc, argument); }

SCM 
gfl_doc_record (Gfl_Info *info)
{
  SCM name, docstring, args, rval, ret, sep;
  int i;
  char d[80];
  name = gh_str02scm(info[0].label);
  docstring = gh_str02scm(info[1].label);
  sep = gh_str02scm(", ");
  
  i = 2;
  args = SCM_EOL;
  while (info[i].label != 0)
    {
      SCM val;
      Gfl_Class * b;
      b = (Gfl_Class *) info[i].id;
      sprintf(d, "(%s) %s", b->name, info[i].label);
      val = gh_str02scm(d);
      args = gh_cons (val, args);
      i = i + 1;
      if (info[i].label != 0)
	args = gh_cons (sep, args);
    }
  i = i + 1;
  rval = SCM_EOL;
  while (info[i].label != 0)
    {
      SCM val;
      Gfl_Class * b;
      b = (Gfl_Class *) info[i].id;
      sprintf(d, "(%s) %s", b->name, info[i].label);
      val = gh_str02scm(d);
      rval = gh_cons (val, rval);
      i = i + 1;
      if (info[i].label != 0)
	rval = gh_cons (sep, rval);
    }

  rval = scm_string_append (gh_reverse(rval));
  args = scm_string_append (gh_reverse(args));
  ret = gh_cons (rval, SCM_EOL);
  ret = gh_cons (args, ret); 
  ret = gh_cons (docstring, ret);
  ret = gh_cons (name, ret);
  return ret;
}

GFL_PROCEDURE_1_4
( gfl_doc,
  "gfl-doc",
  "Returns a document record based on the passed symbol.",
  name,
  symbol,
  name, docstring, args, rvals,
  str, str, str, str)
{
  char *name_;
  Gfl_Info *info;
  name_ = gh_scm2newstr(scm_symbol_to_string(name), NULL);

  info = gfl_find_info (name_);
  if (!info)
    return SCM_BOOL_F;

  free (name_);
  return (gfl_doc_record (info));  
}

GFL_PROCEDURE_1_4
( gfl_doc_index,
  "gfl-doc-index",
  "Returns a document record based on index.",
  N,
  int,
  name, docstring, args, rvals,
  str, str, str, str)
{
  int i, n;
  zAVLCursor x;
  Gfl_Record *y;
  GFL_ASSERT_1 (gfl_doc_index, N);
  n = gh_scm2int(N);

  if (n >= gfl->count) return SCM_BOOL_F;
  if (n < 0) return SCM_BOOL_F;
  
  y = zAVLFirst (&x, gfl);
  for (i = 0; i < n; i++)
    y = zAVLNext (&x);

  if (y)
    return (gfl_doc_record (y->info));
 
  return SCM_BOOL_F;
}

GFL_PROCEDURE_0_1
( gfl_count,
  "gfl-count",
  "Returns the number of procedures in the procedural database.",
  N,
  int)
{
  return gh_int2scm(gfl->count);
}

SCM
gfl_new_procedure_ (Gfl_Info *info, SCM (*fn)())
{
  int n_req_args, n_ret_args;
  Gfl_Info *t;
  for (n_req_args = 2, t = &info[n_req_args]; 
       t->id != 0; 
       n_req_args++, t = &info[n_req_args]);
  for (n_ret_args = n_req_args + 1, t = &info[n_ret_args]; 
       t->id != 0; n_ret_args++, t = &info[n_ret_args]);
  n_ret_args -= (n_req_args + 1);
  n_req_args -= 2;

  gfl_add_ (info, fn, 0);

  return gh_new_procedure( (char *) info[0].label, fn, n_req_args, 0, 0);
}

static Gfl_Class **gfl_classes = 0;
static int gfl_classes_size = 0;

/* Add a class to the master list of classes. */

void
gfl_register_class (Gfl_Class *a)
{
  gfl_classes_size = gfl_classes_size + 1;
  gfl_classes = realloc (gfl_classes, gfl_classes_size*(sizeof(Gfl_Class*)));
  gfl_classes[gfl_classes_size - 1] = a;
}

Gfl_Class *
gfl_find_class (long id)
{
  int i;
  for (i=0; i<gfl_classes_size; i++)
    if (gfl_classes[i]->tag == id) return gfl_classes[i];
  return 0;
}

/* A little honey so we can use argument overloading in c++ */

SCM gfl_new_procedure0 (Gfl_Info *args, SCM (*fn)(void))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure1 (Gfl_Info *args, SCM (*fn)(SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure2 (Gfl_Info *args, SCM (*fn)(SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure3 (Gfl_Info *args, SCM (*fn)(SCM, SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure4 (Gfl_Info *args, SCM (*fn)(SCM, SCM, SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure5 (Gfl_Info *args, SCM (*fn)(SCM, SCM, SCM, SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure6 (Gfl_Info *args, SCM (*fn)(SCM, SCM, SCM, SCM, SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure7 (Gfl_Info *args, SCM (*fn)(SCM, SCM, SCM, SCM, SCM, SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure8 (Gfl_Info *args, SCM (*fn)(SCM, SCM, SCM, SCM, SCM, SCM, SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }
SCM gfl_new_procedure9 (Gfl_Info *args, SCM (*fn)(SCM, SCM, SCM, SCM, SCM, SCM, SCM, SCM, SCM))
  { return gfl_new_procedure_ (args, fn); }

int
gfl_init()
{
  gfl = zAVLAllocTree (gfl_get_key);
  gfl_new_procedure_ (gfl_doc_index_i, gfl_doc_index);
  gfl_new_procedure_ (gfl_doc_i, gfl_doc);
  gfl_new_procedure_ (gfl_count_i, gfl_count);
  return 0;
}
